//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __AREA_COMM_LINK_H__
#define __AREA_COMM_LINK_H__

#include <vector>
#include <string>

#include <Foundation/datatype.h>

class AreaCommLink {
public:
	struct Rect { int head[3], tail[3], size[3]; };

protected:
	const string label;

	const int  gID;           // Group ID
	const int  nID;           // Target Node ID @gID
	const vector<char> aryN;  // array of communication flag

	Rect SlfVxRect;     // Voxel of Self Node
	Rect TgtVxRect;     // Voxel of Target Node
	Rect WrappedRect;   // Voxel Size of Wrapping Inter target and self

	S4D_REAL* buffer;   // communication buffer, size is wrapped I,J,K,E.
	                    //   I ... WrappedRect.size[0]
	                    //   J ... WrappedRect.size[1]
	                    //   K ... WrappedRect.size[2]
	                    //   N ... count of true in aryN

public:
	AreaCommLink(
		const CfgInfo& cfg,
		const string& lbl, const int nid, const int gid,
		const CfgInfo::VoxelDivInfo& slfdiv,
		const CfgInfo::VoxelDivInfo& tgtdiv,
		const std::vector<char> Es);
	~AreaCommLink();

	int Send(const S4D_REAL* src, const bool comm_vc) const;
	int Recv(S4D_REAL* dest, const bool comm_vc)      const;
	bool enable() const;

	int node()  const { return nID; }
	int group() const { return gID; }

	const Rect& GetTargetRect()  const { return TgtVxRect;   }
	const Rect& GetSelfRect()    const { return SlfVxRect;   }
	const Rect& GetWrappedRect() const { return WrappedRect; }

	string toString() const;
};

typedef vector<AreaCommLink*> CommLinks;

/*
int SendRecv(
	S4D_REAL* dest,      const CommLinks& recvlinks,
	const S4D_REAL* src, const CommLinks& sendlinks);
*/
#endif

