//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _SKL_FORTRAN_FUNC_Foundation_H_
#define _SKL_FORTRAN_FUNC_Foundation_H_
#include "Skl.h"

extern "C" {
/* USER WRITE
   if using fortran subroutine,
   Please write fortran subroutine here.
   # The function described here belongs to the name scope of C.
   # Therefore, The identifier of Solver is recommended to be added to
   # the function name to distinguish from other functions.
*/

	void exptim_(SKL_REAL* P1, const SKL_REAL* P0,
				const SKL_REAL* fx, const SKL_REAL* fy, const SKL_REAL* fz,
				const SKL_REAL* Mt, const SKL_REAL* Fm, const SKL_REAL* Qm,
				const SKL_REAL* R,
				const SKL_REAL* S0,
				const SKL_REAL* Vfi,
				const SKL_REAL* Sm,
				const SKL_REAL* dt,
				const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi,
				const int* imax, const int* jmax, const int* kmax,
				const int* vc);

	void exptimab_(int* RhsoInitFlag,
				SKL_REAL* rhs, SKL_REAL* rhso,
				SKL_REAL* P1, const SKL_REAL* P0,
				const SKL_REAL* fx, const SKL_REAL* fy, const SKL_REAL* fz,
				const SKL_REAL* Mt, const SKL_REAL* Fm, const SKL_REAL* Qm,
				const SKL_REAL* R,
				const SKL_REAL* S0,
				const SKL_REAL* Vfi,
				const SKL_REAL* Sm,
				const SKL_REAL* dt,
				const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi,
				const int* imax, const int* jmax, const int* kmax,
				const int* vc);

	void exptimrk_(SKL_REAL* rhs, SKL_REAL* P1, const SKL_REAL* P0,
				const SKL_REAL* Mt, const SKL_REAL* Fm, const SKL_REAL* Qm,
				const SKL_REAL* R,
                                const SKL_REAL* S0,
                                const SKL_REAL* Vfi,
                                const SKL_REAL* Sm,
                                const SKL_REAL* dt,
                                const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi,
                                const int* imax, const int* jmax, const int* kmax,
                                const int* vc);

	void ricscpys3d_(const SKL_REAL* dst, const SKL_REAL* src,
				const int* imax, const int* jmax, const int* kmax,
				const int* vc);

	void ricscpys3dss_(const SKL_REAL* dst, const SKL_REAL* src,
				const int* imax, const int* jmax, const int* kmax,
				const int* dvc, const int* svc);

	void ricscpys3dsendrecv_(const SKL_REAL* dst, const SKL_REAL* src,
				const int* dstvc, const int* srcvc,
				const int* imax, const int* jmax, const int* kmax,
				const int* vc);

	void ricscpys3dareacomm_(const SKL_REAL* dst,
				const int* dimax, const int* djmax, const int* dkmax,
				const int* dvc,
				const SKL_REAL* src,
				const int* simax, const int* sjmax, const int* skmax,
				const int* svc,
				const int* wi, const int* wj, const int* wk, const int* si, const int* sj, const int* sk, const int* vc);

	void ricscpys3dcoef_(const SKL_REAL* dst, const SKL_REAL* src,
                                const int* imax, const int* jmax, const int* kmax,
                                const int* vc, SKL_REAL* coef);

	void ricsadds3dcoef_(const SKL_REAL* dst, const SKL_REAL* src,
                                const int* imax, const int* jmax, const int* kmax,
                                const int* vc, SKL_REAL* coef);

	void makevolume_(const SKL_REAL *Vf,
				const int *imax, const int *jmax, const int *kmax,
				const int *vc,
				int *xmin, int *xmax, int *ymin, int *ymax,
				int *zmin, int *zmax,
				const int *hix, const int *hiy, const int *hiz, int *mask);

	void setopenarearatio_(const SKL_REAL *Vf,
				const SKL_REAL *Ax, const SKL_REAL *Ay, const SKL_REAL *Az,
				const int *vc,
				const int *imax, const int *jmax, const int *kmax);

	void sschkabslt_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkabsle_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkabsgt_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkabsge_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkratiolt_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkratiole_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkratiogt_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

	void sschkratioge_(int *flag, SKL_REAL *threshold,
				const SKL_REAL *SSFLD, const SKL_REAL *P0, const SKL_REAL *VOF,
				const int *imax, const int *jmax, const int *kmax,
				const int *ssvc, const int *p0vc);

}

#endif // _SKL_FORTRAN_FUNC_Foundation_H_

