/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2007-2010
 *
 */

#ifdef MAKE_MODEL

/*
#include <sstream>
#include <iomanip>

#include <stdlib.h>
#include <math.h>
*/

#include <Foundation/const.h>
#include <Foundation/error.h>
#include <Foundation/util.h>
#include <Foundation/api.h>

#include "SklSolverFoundation.h"

int SklSolverFoundation::MakeMatByIndex(int xmin, int xmax, int ymin, int ymax,
					int zmin, int zmax, size_t matid)
{
	const char *name = cfg.MatTbl[matid]->name.c_str();
	MESSAGE(string("Make Volume data of ") + name, 0);
	MESSAGE(UtilStrFormat("  X index(min, max) = (%d, %d)", xmin, xmax), 0);
	MESSAGE(UtilStrFormat("  Y index(min, max) = (%d, %d)", ymin, ymax), 0);
	MESSAGE(UtilStrFormat("  Z index(min, max) = (%d, %d)", zmin, zmax), 0);

	const int vc  = VC;

	const int hix = cfg.Pdiv.head[0];
	const int hiy = cfg.Pdiv.head[1];
	const int hiz = cfg.Pdiv.head[2];

	const int I   = cfg.Pdiv.size[0];
	const int J   = cfg.Pdiv.size[1];
	const int K   = cfg.Pdiv.size[2];

	const SKL_REAL *Vf = UtilGetS4DN(MatVfFld, matid);
	const SKL_REAL *Ax = UtilGetS4DN(MatAreaX, matid);
	const SKL_REAL *Ay = UtilGetS4DN(MatAreaY, matid);
	const SKL_REAL *Az = UtilGetS4DN(MatAreaZ, matid);

	size_t sz = (I+2*vc)*(J+2*vc)*(K+2*vc);
	int *mask = new int[sz];

	makevolume_(Vf,&I,&J,&K,&vc,&xmin,&xmax,&ymin,&ymax,&zmin,&zmax,
			&hix,&hiy,&hiz,mask);

	delete [] mask;

	MatVfFld->CommBndCell(MatVfFld->GetVCellSize());

	setopenarearatio_(Vf,Ax,Ay,Az,&vc,&I,&J,&K);

	MatAreaX->CommBndCell(MatAreaX->GetVCellSize());
	MatAreaY->CommBndCell(MatAreaY->GetVCellSize());
	MatAreaZ->CommBndCell(MatAreaZ->GetVCellSize());

	return 1;
}

int SklSolverFoundation::MakeMatGLC2Hx()
{
	MESSAGE("Make GLC2Hx model Material ...", 0);

	// Make M1 Volume
	size_t matid = 0;

	SKL_REAL xscale = SKL_REAL(cfg.size[0]) / 16.0;
	SKL_REAL yscale = SKL_REAL(cfg.size[1]) / 16.0;
	SKL_REAL zscale = SKL_REAL(cfg.size[2]) / 16.0;

	// int xmin = 4 * xscale;	int xmax = 11 * xscale;
	// int ymin = 4 * yscale;	int ymax = 11 * yscale;
	// int zmin = 4 * zscale;	int zmax = 11 * zscale;

	SKL_REAL memMin =  4.5;
	SKL_REAL memMax = 11.5;
	int xmin = int(memMin * xscale);
	int xmax = int(memMax * xscale);

	int ymin = xmin;	int zmin = xmin;
	int ymax = xmax;	int zmax = xmax;

	MESSAGE(UtilStrFormat("Model Scale (X, Y, Z) = (%e, %e, %e)", xscale,yscale,zscale), 0);

	MakeMatByIndex(xmin, xmax, ymin, ymax, zmin, zmax, matid);

	return 1;
}

int SklSolverFoundation::MakeMatRect()
{
	MESSAGE("Make model Material ...", 0);

	const CfgElem* DomainInfo = SklCfgGetTop(DOMAININFO);
	const CfgElem* cell       = DomainInfo->GetElemFirst("cell");

	const CfgParam* param;
	int iparam;

	size_t matid = 0;

	while(cell){
		const CfgElem* mat = cell->GetElemFirst();
		while(mat){
			if( matid != cfg.CmpMatId ){
				int xmin = 0;
				int ymin = 0;
				int zmin = 0;
				int xmax = cfg.size[0] - 1;
				int ymax = cfg.size[1] - 1;
				int zmax = cfg.size[2] - 1;

				iparam = 0;
				// x min
				if( param = mat->GetParamFirst("xmin") ){
					if( param->GetData(&iparam) ){
						xmin = iparam;
					}
				}

				iparam = 0;
				// x max
				if( param = mat->GetParamFirst("xmax") ){
					if( param->GetData(&iparam) ){
						xmax = iparam;
					}
				}

				iparam = 0;
				// y min
				if( param = mat->GetParamFirst("ymin") ){
					if( param->GetData(&iparam) ){
						ymin = iparam;
					}
				}

				iparam = 0;
				// y max
				if( param = mat->GetParamFirst("ymax") ){
					if( param->GetData(&iparam) ){
						ymax = iparam;
					}
				}

				iparam = 0;
				// z min
				if( param = mat->GetParamFirst("zmin") ){
					if( param->GetData(&iparam) ){
						zmin = iparam;
					}
				}

				iparam = 0;
				// z max
				if( param = mat->GetParamFirst("zmax") ){
					if( param->GetData(&iparam) ){
						zmax = iparam;
					}
				}

				// Make Material
				MakeMatByIndex(xmin, xmax, ymin, ymax, zmin, zmax, matid);
			}
			matid++;

			mat = cell->GetElemNext(mat, NULL);
		}
		cell = DomainInfo->GetElemNext(cell, "cell");
	}

	return 1;
}

#endif // MAKE_MODEL
