//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include <sstream>

#include <Foundation/ecellif/ecellif.h>
#include <Foundation/util.h>
#include <Foundation/const.h>
#include <Foundation/error.h>

#include <UserDefined/UserDefinedFunction.h>

#include "SklSolverFoundation.h"


bool
SklSolverFoundation::SklSolverPost() {
/* USER WRITE
   Please write solver post code.
*/
    const size_t mats = cfg.MatCount;
    const size_t mols = cfg.MolCount;

	try {
	///////////////////////////////////////////////////////////////
	// Finalize Local Solvers
	UserDefinedFunctionPost();

	///////////////////////////////////////////////////////////////
	// Finalize Local Solvers
	mtbsolv.MetaboSolvPost();
	difsolv.DiffusionPost();
	membsolv.MembranePost();

#ifdef ENABLE_MEMBRANEPOTENTIAL
	potentialsolv.PotentialPost();
#endif

#ifdef ENABLE_ADVECTION
	advectionsolv.AdvectionPost();
#endif

#ifdef _COST_BMT
	///////////////////////////////////////////////////////////////
	// delete allocated objects
	
#ifdef SPHERE175
	para_cmp->Barrier();	//2009.12.07
#else
	ParaMngr.Barrier();
#endif
	
	char mloop[81], metabo[81], diffus[81], memb[81], user[81], expPhi[81], io[81], all[81];
	const double allClock   = clock_all;
	const double ioClock    = clock_io;
	const double expClock   = clock_exptim;
	const double mlpClock   = clock;
	const double mlpCommClk = clock_comm;
	const double mlpWaitClk = clock_wait;
	const double mtbClock   = mtbsolv.GetClock();
	const double mtbCommClk = mtbsolv.GetCommClk();
	const double mtbWaitClk = mtbsolv.GetWaitClk();
	const double difClock   = difsolv.GetClock();
	const double difCommClk = difsolv.GetCommClk();
	const double difWaitClk = difsolv.GetWaitClk();
	const double mmbClock   = membsolv.GetClock();
	const double mmbCommClk = membsolv.GetCommClk();
	const double mmbWaitClk = membsolv.GetWaitClk();
	const double UserClock  = clock_user;

#ifdef ENABLE_MEMBRANEPOTENTIAL
	char poten[81];
	const double potClock   = potentialsolv.GetClock();
	const double potCommClk = potentialsolv.GetCommClk();
	const double potWaitClk = potentialsolv.GetWaitClk();
#endif // ENABLE_MEMBRANEPOTENTIAL

#ifdef ENABLE_ADVECTION
	char advec[81];
	const double advClock   = advectionsolv.GetClock();
	const double advCommClk = advectionsolv.GetCommClk();
	const double advWaitClk = advectionsolv.GetWaitClk();
#endif // ENABLE_ADVECTION

#ifdef _MEASURE_WAIT
	sprintf(mloop, "Loop           : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			mlpClock, mlpCommClk, mlpWaitClk);
	sprintf(metabo,"* Metabolome   : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			mtbClock, mtbCommClk, mtbWaitClk);
	sprintf(diffus,"* Diffusion    : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			difClock, difCommClk, difWaitClk);
	sprintf(memb,  "* Membrane     : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			mmbClock, mmbCommClk, mmbWaitClk);
#ifdef ENABLE_MEMBRANEPOTENTIAL
	sprintf(poten, "* MemPotential : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			potClock, potCommClk, potWaitClk);
#endif // ENABLE_MEMBRANEPOTENTIAL

#ifdef ENABLE_ADVECTION
	sprintf(advec, "* Advection    : %9.3f[s] (%8.3f[s] comm, %8.3f[s] wait)",
			advClock, advCommClk, advWaitClk);
#endif // ENABLE_ADVECTION

#else //_MEASURE_WAIT
	sprintf(mloop, "Loop           : %9.3f[s] (%8.3f[s])", mlpClock, mlpCommClk);
	sprintf(metabo,"* Metabolome   : %9.3f[s] (%8.3f[s])", mtbClock, mtbCommClk);
	sprintf(diffus,"* Diffusion    : %9.3f[s] (%8.3f[s])", difClock, difCommClk);
	sprintf(memb,  "* Membrane     : %9.3f[s] (%8.3f[s])", mmbClock, mmbCommClk);
#ifdef ENABLE_MEMBRANEPOTENTIAL
	sprintf(poten, "* MemPotential : %9.3f[s] (%8.3f[s])", potClock, potCommClk);
#endif // ENABLE_MEMBRANEPOTENTIAL
#ifdef ENABLE_ADVECTION
	sprintf(advec, "* Advection    : %9.3f[s] (%8.3f[s])", advClock, advCommClk);
#endif // ENABLE_ADVECTION
#endif //_MEASURE_WAIT

	sprintf(expPhi,"* Exptim       : %9.3f[s]", expClock);
	sprintf(io,    "* IO           : %9.3f[s]", ioClock);
	sprintf(user,  "* UserCode     : %9.3f[s]", UserClock);
	sprintf(all,   "All            : %9.3f[s]", allClock);

	stringstream ss;
	ss << "load balance #" << cfg.grpAll.nid << endl;
	ss << mloop  << endl;
	ss << metabo << endl;
	ss << diffus << endl;
	ss << memb   << endl;
#ifdef ENABLE_MEMBRANEPOTENTIAL
	ss << poten  << endl;
#endif // ENABLE_MEMBRANEPOTENTIAL
#ifdef ENABLE_ADVECTION
	ss << advec  << endl;
#endif // ENABLE_ADVECTION
	ss << expPhi << endl;
	ss << io     << endl;
	ss << user   << endl;
	ss << all    << endl;
	MESSAGE(ss.str(), -1);

#ifdef SPHERE175
	para_cmp->Barrier();
#else
	ParaMngr.Barrier();
#endif
#endif  // _COST_BMT

	FinalizeECellIf();
	CloseLogFile();

	///////////////////////////////////////////////////////////////
	// on error
	} catch (std::exception& e) {
		MSG_ERROR(e.what());
		if (cfg.Parallel) {

#ifdef SPHERE175
			para_cmp->Abort(1);	//2009.12.07
#else
			ParaMngr.Abort(1);
#endif

		}
	}

	return true;
}

