//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __API_H__
#define __API_H__

#include <Skl.h>
#include "datatype.h"
#include "util.h"

//////////////////////////////////////////////////////////////////////////////
// User Extension Parameter
int RicsGetExtParamInt(const char* paramName);
SKL_REAL RicsGetExtParamReal(const char* paramName);
const char* RicsGetExtParamString(const char* paramName);

//////////////////////////////////////////////////////////////////////////////
// User External Data
bool RicsLoadCSV(const char* fileName, RealPairVector* table);
SKL_REAL RicsGetValueFromTable(const SKL_REAL x, const RealPairVector* table, const SKL_REAL offst);

//////////////////////////////////////////////////////////////////////////////
// Domain
void RicsGetVoxelWidth(int* id, int* jd, int* kd);

//////////////////////////////////////////////////////////////////////////////
// Time
unsigned int RicsGetCurrentStep();
SKL_REAL RicsGetCurrentTime();
SKL_REAL RicsGetDeltaT();

//////////////////////////////////////////////////////////////////////////////
// Vfm
int RicsGetMaterialVolumeRate(const char* name, SKL_REAL* value, int i, int j, int k);

/////////////////////////////////////////////////////////////////////////////
// phi
int RicsGetPhi(const char* mat, const char* mol, SKL_REAL* conc, int i, int j, int k);
int RicsSetPhi(const char* mat, const char* mol, const SKL_REAL conc, int i, int j, int k);

//////////////////////////////////////////////////////////////////////////////
// source term
int RicsAddSrc(const char* mat, const char* mol, const SKL_REAL src, int i, int j, int k);

//////////////////////////////////////////////////////////////////////////////
// FlipFlop
int RicsFlipFlop(const char* mat0, const char* mat1, const char* mol, const SKL_REAL D, int i, int j, int k);

//////////////////////////////////////////////////////////////////////////////
// for membrane function
// penetration ///////////////////////////////////////////////////////////////
// Kpene
int RicsGetMembraneKPene(const char* name, const char* mol, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKPene(const char* name, const char* mol, const SKL_REAL value,  int i, int j, int k);

// channel ///////////////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwCha(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwCha(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyCha(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyCha(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenCha(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenCha(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// K
int RicsGetMembraneKCha(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKCha(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Ke
int RicsGetMembraneKeCha(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKeCha(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// pump //////////////////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxATP
int RicsGetMembraneVmaxAtpPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxAtpPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmATP
int RicsGetMembraneKmAtpPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmAtpPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxGTP
int RicsGetMembraneVmaxGtpPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxGtpPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmGTP
int RicsGetMembraneKmGtpPum(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmGtpPum(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// symport channel ///////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwChaSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwChaSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyChaSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyChaSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenChaSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenChaSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// K
int RicsGetMembraneKChaSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKChaSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Ke
int RicsGetMembraneKeChaSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKeChaSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// symport pump //////////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxATP
int RicsGetMembraneVmaxAtpPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxAtpPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmATP
int RicsGetMembraneKmAtpPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmAtpPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxGTP
int RicsGetMembraneVmaxGtpPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxGtpPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmGTP
int RicsGetMembraneKmGtpPumSym(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmGtpPumSym(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// antiport channel //////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwChaAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwChaAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyChaAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyChaAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenChaAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenChaAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// K
int RicsGetMembraneKChaAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKChaAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Ke
int RicsGetMembraneKeChaAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKeChaAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// antiport pump /////////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxATP
int RicsGetMembraneVmaxAtpPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxAtpPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmATP
int RicsGetMembraneKmAtpPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmAtpPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// VmaxGTP
int RicsGetMembraneVmaxGtpPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneVmaxGtpPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// KmGTP
int RicsGetMembraneKmGtpPumAnti(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKmGtpPumAnti(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// gap janction //////////////////////////////////////////////////////////////
// Sw
int RicsGetMembraneSwGap(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneSwGap(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// fuzzy
int RicsGetMembraneFuzzyGap(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneFuzzyGap(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Den
int RicsGetMembraneDenGap(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneDenGap(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// K
int RicsGetMembraneKGap(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKGap(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

// Ke
int RicsGetMembraneKeGap(const char* name, const char* mol, int id, SKL_REAL* value, int i, int j, int k);
int RicsSetMembraneKeGap(const char* name, const char* mol, int id, const SKL_REAL value,  int i, int j, int k);

//////////////////////////////////////////////////////////////////////////////
// memory info from system
int RicsGetSystemMemoryInfo(SystemMemoryInfo& info);
int RicsShowSystemMemoryInfo();
int RicsSaveSystemMemoryInfo(const char* prefix, const char* udstring, const SystemMemoryInfo& info);

//////////////////////////////////////////////////////////////////////////////
// check values
int RicsCheckPhiValue(const char* mat, const char* mol, CheckValueInfo& info, SKL_REAL bottom = -REAL_MAX, SKL_REAL upper  =  REAL_MAX);

int RicsSaveCheckValueInfo(const char* prefix, const char* udstring, const CheckValueInfo& info);

int RicsCheckPhiValueByDivision(const char* mat, const char* mol, CheckValueInfo& info, SKL_REAL bottom = -REAL_MAX, SKL_REAL upper  =  REAL_MAX);
#if 1
int TestSetPhi(const char* mat, const char* mol, const SKL_REAL conc, int i, int j, int k);
#endif

/////////////////////////////////////////////////////////////////////////////
// number of molecules

// return unit is pieces
int RicsGetNumberOfMolecules(const char* mat, const char* mol, SKL_REAL* nmols, int i, int j, int k);

// return unit is mole number
int RicsGetMoleNumber(const char* mat, const char* mol, SKL_REAL* molenum, int i, int j, int k);

/////////////////////////////////////////////////////////////////////////////
// number of total molecules

// return unit is pieces
int RicsGetTotalNumberOfMolecules(const char*mat, const char* mol, SKL_REAL* nmols);

// return unit is mole number
int RicsGetTotalMoleNumber(const char* mat, const char* mol, SKL_REAL* molenum);

#endif // __API_H__
