//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __EMLMODEL_HPP__
#define __EMLMODEL_HPP__

#ifndef _WITHOUT_ECELL

#include <libxml/tree.h>
//#include <libecs/libecs.hpp>
#include <libecs/Polymorph.hpp>

#include <string>
#include <vector>
#include <map>

template <typename T, T sep>
class strcon {
	const std::string str;
public:
	strcon(const std::string& f, const std::string& s)
		: str(f + sep + s) { }

	strcon(const std::string& f, const std::string& s, const std::string& t)
		: str(f + sep + s + sep + t) { }

	const std::string toString() const { return str; }
	operator const std::string() const { return str; }
};

static const char id_sep   = ':';
static const char path_sep = '/';

typedef strcon<const char, id_sep>   makeid;
typedef strcon<const char, path_sep> mkpath;

class EmlModel {
public:
	struct stepper {
		std::string cls;
		std::string id;
	};
	struct entity {
		std::string type;
		std::string cls;
		std::string path;
		std::string id;
	};

	typedef std::vector<stepper>   StepperList;
	typedef std::vector<entity>    EntityList;
	typedef std::map<
				std::string,
				libecs::Polymorph> PropertyMap;

	typedef std::vector<std::string> StepperNameList;
	typedef std::vector<std::string> EntityNameList;

private:
	std::string filename;

	StepperList stepper_list;
	EntityList  entity_list;
	PropertyMap property_map;
	//
	EntityList system_list;
	EntityList variable_list;
	EntityList process_list;
	//

	void find_eml(xmlNodePtr& eml);
	void find_stepper(xmlNodePtr& stp);
	void find_entity(xmlNodePtr& ent, const std::string& path);
	void find_property(xmlNodePtr& p, const std::string& id);
	libecs::Polymorph find_value(xmlNodePtr& p, const std::string& id);

public:
	EmlModel();
	EmlModel(const char* emlfile);
	~EmlModel();

	void open(const char* emlfile);
	const char* getEmlFileName() const;

	void Dump() const;

	operator const StepperList&() const;
	operator const EntityList&()  const;
	operator const PropertyMap&() const;
};

#endif // _WITHOUT_ECELL
#endif //__EMLMODEL_HPP__

