//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include <string>

#include <Foundation/error.h>
#include <Foundation/util.h>

#ifndef _WITHOUT_ECELL

#include "ecellif.h"

using std::string;

void InitECellIf()
{
#ifndef _NO_MESSAGE
	MESSAGE("Initialize libecs...");
#else
	MESSAGE("Initialize libecs...",0);
#endif // _NO_MESSAGE
#if ECELL_VERSION_INT >= VERSION_INT(3,1,106)
	libecs::initialize();
#endif
#ifndef _NO_MESSAGE
	MESSAGE(UtilStrFormat("E-CELL Version: %s",ECELL_VERSION_STRING));
#else
	MESSAGE(UtilStrFormat("E-CELL Version: %s",ECELL_VERSION_STRING),0);
#endif // _NO_MESSAGE
}

void FinalizeECellIf()
{
#if ECELL_VERSION_INT >= VERSION_INT(3,1,106)
	libecs::finalize();
#endif
}

//string poly2string(const libecs::Polymorph p)
string poly2string(const libecs::Polymorph p)
{
	string ret;
#ifdef ECELL_3_1
	if (p.getType() == libecs::Polymorph::POLYMORPH_VECTOR) {
		const libecs::PolymorphVector v = p.asPolymorphVector();
#else
	if (p.getType() == libecs::PolymorphValue::TUPLE) {
		const libecs::PolymorphVector v = p.as<libecs::PolymorphVector>();
#endif //ECELL_3_1
		ret = "(";
		for (libecs::PolymorphVector::const_iterator
				i = v.begin(); i != v.end(); i++ ) {
			if (i != v.begin()) { ret += ", "; }
			ret += poly2string(*i);
		}
		ret += ")";
#ifdef ECELL_3_1
	} else if (p.getType() == libecs::Polymorph::STRING) {
		ret += UtilStrFormat("'%s'",p.asString().c_str());
	} else if (p.getType() == libecs::Polymorph::REAL) {
		ret += UtilStrFormat("%e",p.asReal());
	} else if (p.getType() == libecs::Polymorph::INTEGER) {
		ret += UtilStrFormat("%d",p.asInteger());
	} else {
		ret += p.asString() + "<NONE>";
#else
	} else if (p.getType() == libecs::PolymorphValue::STRING) {
		ret += UtilStrFormat("'%s'",p.as<libecs::String>().c_str());
	} else if (p.getType() == libecs::PolymorphValue::REAL) {
		ret += UtilStrFormat("%e",p.as<libecs::Real>());
	} else if (p.getType() == libecs::PolymorphValue::INTEGER) {
		ret += UtilStrFormat("%d",p.as<libecs::Integer>());
	} else {
		ret += p.as<libecs::String>() + "<NONE>";
#endif //ECELL_3_1
	} 

	return ret;
}

#endif // _WITHOUT_ECELL

