//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include <SklSolverBase.h>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <Foundation/util.h>

#include "error.h"

Error::Error(const char* msg):        _msg(msg) {}
Error::Error(const std::string& msg): _msg(msg) {}
Error::~Error() throw() { }
const char* Error::what() const throw() { return _msg.c_str(); }

void MESSAGE(const std::string& msg, const int nid, const bool show_nid, std::ostream& os)
{

#ifdef SPHERE175
	//2009.12.07
	const CfgInfo *cfg = UtilGetCfg();
	SklParaComponent *para_cmp = cfg->solver->SklGetParaComponent();
	SklParaManager *para_mng = para_cmp->GetParaManager();
#endif

#ifdef SPHERE175
	const int nID = (para_cmp->IsParallel())? para_cmp->GetMyID(0): 0;	//2009.12.08
#else
	const int nID = (ParaMngr.IsParallel())? ParaMngr.GetMyID(0): 0;
#endif

	ostream* p = &os;

	if ((nID != nid) && (nid != -1)) { return; }
	do {
		if (show_nid) { *p << "[" << nID << "] "; }
		*p << msg << endl;
	} while (p = p->tie());
}

void MSG_WARNING(const std::string& msg, const bool flg, const int nid)
{
	if (!flg) return;
	MESSAGE(string("WARNING: ") + msg, nid, true, std::cerr);
}

void MSG_ERROR(const std::string& msg, const bool flg, const int nid)
{
	if (!flg) return;
	MESSAGE(string("ERROR:   ") + msg, nid, true, std::cerr);
}


