//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __ERROR_H__
#define __ERROR_H__

#include <exception>
#include <string>
#include <sstream>
#include <iostream>

#include <Foundation/util.h>

#define ERROR_PLACE UtilStrFormat("%s:%d",__FILE__,__LINE__)
#define ERROR(msg) \
	throw Error(std::string(msg) + "\n" + (ERROR_PLACE));
#define TRY(stmt, msg) \
	if (!(stmt)) { throw Error(std::string(msg) + "\n" + (ERROR_PLACE)); }

class Error : public std::exception {
	const std::string _msg;

public:
	Error(const char* msg);
	Error(const std::string& msg);

	virtual ~Error() throw();

	virtual const char* what() const throw();
};

void MESSAGE    (const std::string& msg, const int nid = -1, const bool show_nid = false, std::ostream& os = std::cout);
void MSG_WARNING(const std::string& msg, const bool flg = true, const int nid = -1);
void MSG_ERROR  (const std::string& msg, const bool flg = true, const int nid = -1);

#endif // __ERROR_H__

