//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __UTIL_H__
#define __UTIL_H__

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <cfloat>

#include <SklUtil.h>
#include <SklSolverBase.h>

#include <Foundation/datatype.h>
#include <Foundation/error.h>

#define grater(l,r) (((l)>(r))? (l): (r))
#define lesser(l,r) (((l)<(r))? (l): (r))

#ifdef REAL_IS_DOUBLE
	#define REAL_MAX DBL_MAX
	#define REAL_MIN DBL_MIN
#else
	#define REAL_MAX FLT_MAX
	#define REAL_MIN FLT_MIN
#endif
#define REAL_VALID(v)   ((-REAL_MAX < (v)) && ((v) < REAL_MAX))
#define REAL_INVALID(v) ((-REAL_MAX > (v)) || ((v) > REAL_MAX))

#ifndef _DBLV
	#ifdef _DEBUG
		#define _DBLV 5
	#else
		#define _DBLV 0
	#endif
#endif

#if _DBLV >= 1
	#define DB1(prog) prog
#else
	#define DB1(prog)
#endif // _DBLV => 1
#if _DBLV >= 2
	#define DB2(prog) prog
#else
	#define DB2(prog)
#endif // _DBLV => 2
#if _DBLV >= 3
	#define DB3(prog) prog
#else
	#define DB3(prog)
#endif // _DBLV => 3
#if _DBLV >= 4
	#define DB4(prog) prog
#else
	#define DB4(prog)
#endif // _DBLV => 4
#if _DBLV >= 5
	#define DB5(prog) prog
#else
	#define DB5(prog)
#endif // _DBLV => 5
#if _DBLV < 0
        #define DBX(prog) prog
#else
        #define DBX(prog)
#endif

#ifdef _COST_BMT
#define BEGIN_CLOCK(LAP) \
	double LAP = SklSolverBase::SklGetTime();
#define END_CLOCK(CLK,LAP) \
	CLK += SklSolverBase::SklShowIntervalTime(&LAP);
#else
#define BEGIN_CLOCK(LAP)
#define END_CLOCK(CLK,LAP)
#endif

#ifdef _MEASURE_WAIT
    #ifndef _COST_BMT
        #define _COST_BMT
    #endif
    #define MEASURE_WAIT_TIME(CLK, BARRIER)  \
    {   BEGIN_CLOCK(LAP_WAIT)	    \
  	    BARRIER;				    \
		END_CLOCK(CLK, LAP_WAIT)    \
	}
#else
    #define MEASURE_WAIT_TIME(CLK, BARRIER)
#endif

void InitUtilEnv(CfgInfo* _cfg);
CfgInfo* UtilGetCfg();

//////////////////////////////////////////////////////////
// extension parameter
int UtilGetExtParamInt(const char* paranName);
SKL_REAL UtilGetExtParamReal(const char* paranName);
const char* UtilGetExtParamString(const char* paranName);

//////////////////////////////////////////////////////////
// formated string
std::string UtilStrFormat(const char* format, ... );

//////////////////////////////////////////////////////////
// Sub 3D array head at 4D array
SKL_REAL* UtilGetS4DN(S4D_REAL* s4d, const int n = 0);
const SKL_REAL* UtilGetS4DN(const S4D_REAL* s4d, const int n = 0);

//////////////////////////////////////////////////////////
// Allocation
S4D_REAL* UtilAllocS4DReal(const char* label, const vxsize_t* size, const int vc, const int gid);
S3D_REAL* UtilAllocS3DReal(const char* label, const vxsize_t* size, const int vc, const int gid);
S4D_INT* UtilAllocS4DInt(const char* label, const vxsize_t* size, const int vc, const int gid);
S4D_UINT* UtilAllocS4DUInt(const char* label, const vxsize_t* size, const int vc, const int gid);

//////////////////////////////////////////////////////////
// SPH Load
int UtilLoadSph(const string& attr, S4D_REAL* ary, const int N, const int gid);
int UtilLoadSph(const char* attr, S4D_REAL* ary, const int N, const int gid);
int UtilLoadSph(const string& attr, S3D_REAL* ary, const int gid);
int UtilLoadSph(const char* attr, S3D_REAL* ary, const int gid);
//int UtilLoadSph(const char* attr, V3D_REAL* ary, const int gid);

//////////////////////////////////////////////////////////
// ID
const Material*   UtilGetMaterialId(const string& name);
const Molecular*  UtilGetMolecularId(const string& name);
const Equation*   UtilGetEquationId(const string& name);
const Metabolism* UtilGetMetabolismId(const string& name);
const Membrane*   UtilGetMembraneId(const string& name);
const MolMemb*    UtilGetMolMembId(const string& name);
const Membarea*   UtilGetMembareaId(const string& name);

const Equation* UtilGetEquationId(const Molecular* ml, const Material* mt);
const Membrane* UtilGetMembraneId(const Material* m, const Material* n);
const MolMemb*  UtilGetMolMembId(const Molecular* ml, const Membrane* mb);
const Membarea* UtilGetMembareaId(const Material* mid, const Material* nid);

const penetrater* UtilGetPenetraterId(const string& name);
const ChTranser*  UtilGetChTranserId(const string& name);
const PmTranser*  UtilGetPmTranserId(const string& name);
const CSTranser*  UtilGetCSTranserId(const string& name);
const PSTranser*  UtilGetPSTranserId(const string& name);
const CATranser*  UtilGetCATranserId(const string& name);
const PATranser*  UtilGetPATranserId(const string& name);
const GpTranser*  UtilGetGpTranserId(const string& name);

const MolMemb* UtilAddNewMolMembId(const Molecular* g, const Membrane* mn);

//////////////////////////////////////////////////////////
// Register Array
bool UtilRegArrayS4DReal(const string& name, S4D_REAL* ary, int N);
S4DRN UtilGetRegistedArrayS4DReal(const string& name);

//////////////////////////////////////////////////////////
// extension parameter access function
const ExtParam* UtilGetExtParam(const string& name);

int UtilGetExtParamInt(const string& name);
SKL_REAL UtilGetExtParamReal(const string& name);
const char* UtilGetExtParamStr(const string& name);

int UtilGetExtParamID(const string& name);
const char* UtilGetExtParamComment(const string& name);

bool UtilClearRealArray(SklAray<SKL_REAL>* ary, const SKL_REAL val=0);
bool UtilClearIntArray(SklAray<int>* ary, const int val = 0);
bool UtilClearUIntArray(SklAray<unsigned int>* ary, const unsigned int val = 0);

bool UtilCheckIdxPdiv(int* zi, int* zj, int* zk, const int i, const int j, const int k, const int vc = 0);

//////////////////////////////////////////////////////////
// external table file (CSV)
bool UtilLoadCSV(const string& filename, RealPairVector* table, const char sep = ',');

//////////////////////////////////////////////////////////
// interpolation
SKL_REAL UtilLinearInterpolation(const SKL_REAL x, const SKL_REAL x1, const SKL_REAL y1, const SKL_REAL x2, const SKL_REAL y2);
SKL_REAL UtilGetValueFromTable(const SKL_REAL x, const RealPairVector* table, const SKL_REAL offst);

//////////////////////////////////////////////////////////
 
int UtilSetMembParam(const string& fname, const membfunction* p,
						const int i, const int j, const int k,
						const SKL_REAL value);
int UtilGetMembParam(const string& fname, const membfunction* p,
						const int i, const int j, const int k,
						SKL_REAL* value);

//////////////////////////////////////////////////////////
SKL_REAL UtilGetFieldSum(const S3D_REAL* s3d);
SKL_REAL UtilGetFieldSum(const S4D_REAL* s4d, const int N);
SKL_REAL UtilGetMassSum(const Equation* e, const S4D_REAL* p, const S4D_REAL* vf);

//////////////////////////////////////////////////////////
template <class T>
bool UtilClearArray(SklAray<T>* ary, const T val = 0)
{
	T* data = reinterpret_cast<T*>(ary->GetDataP());
	const int size = ary? ary->GetArrayLength(): 0;
	if (!size) { return true; }
	for (int i = 0; i < size; i++) {
		data[i] = val;
	}
	return true;
}

template <class T>
bool UtilClearS4DN(SklScalar4D<T>* ary, const size_t N, const T val=0)
{
	if (!ary) { return false; }
	if (ary->GetSize()[3] <= N) { return false; }
	for (int k = 0; k < ary->_GetSize()[2]; k++) {
	for (int j = 0; j < ary->_GetSize()[1]; j++) {
	for (int i = 0; i < ary->_GetSize()[0]; i++) {
		ary->_val(i,j,k,N) = val;
	}}}
	return true;
}

template <class T>
bool UtilLoadSphOrEmmToS4DN(SklScalar4D<T>* ary, const string& attr, const T val, const size_t N, const int gid)
{
	if (UtilGetCfg()->solver->SklCfgCheckInFile(attr.c_str())) {
		UtilLoadSph(attr,ary,N,gid);
	} else {
		UtilClearS4DN(ary,N,val);
	}
	return true;
}

template <class T>
bool UtilLoadSphOrEmmToS3D(SklScalar3D<T>* ary, const string& attr, const T val, const int gid)
{
	if (UtilGetCfg()->solver->SklCfgCheckInFile(attr.c_str())) {
		UtilLoadSph(attr,ary,gid);
	} else {
		UtilClearArray(ary,val);
	}
	return true;
}
//////////////////////////////////////////////////////////

template <class T>
inline std::string string_cast(const T& val, std::ios::fmtflags fmt = std::ios::dec)
{
	stringstream ss;
	string s;
	
	ss.setf(fmt);
	ss << val;
	ss >> s;

	return s;
}

template <class T>
inline std::string S3D2string(const SklScalar3D<T>* fld, const bool vvc = false)
{
	const int vc = static_cast<int>((vvc)? fld->GetVCellSize(): 0);
	const int sz[] = {
		fld->GetSize()[0] + (vvc? vc: 0),
		fld->GetSize()[1] + (vvc? vc: 0),
		fld->GetSize()[2] + (vvc? vc: 0),
	};

	//
	std::stringstream os;
	os << sz[0] << ',' << sz[1] << ',' << sz[2] << ":" << vc << std::endl;

	for (int k = -vc; k < sz[2]; k++) {
		os << "k = " << k << std::endl;
		for (int j = -vc; j < sz[1]; j++) {
			for (int i = -vc; i < sz[0]; i++) {
				const T& v = fld->val(i,j,k);
				os << UtilStrFormat(" %+1.7e", v);
			}
			os << std::endl;
		}
		os << std::endl;
	}

	return os.str();
};

template <class T>
inline std::string S4D2string(const SklScalar4D<T>* fld, const int n, const bool vvc = false)
{
	const int vc = static_cast<int>((vvc)? fld->GetVCellSize(): 0);
	const int sz[] = {
		fld->GetSize()[0] + (vvc? vc: 0),
		fld->GetSize()[1] + (vvc? vc: 0),
		fld->GetSize()[2] + (vvc? vc: 0),
	};

	//
	std::stringstream os;
	os << sz[0] << ',' << sz[1] << ',' << sz[2] << ":" << vc << std::endl;

	for (int k = -vc; k < sz[2]; k++) {
		os << "k = " << k << std::endl;
		for (int j = -vc; j < sz[1]; j++) {
			for (int i = -vc; i < sz[0]; i++) {
				const T& v = fld->val(i,j,k,n);
				os << UtilStrFormat(" %+1.7e", v);
			}
			os << std::endl;
		}
		os << std::endl;
	}

	return os.str();
};

template <class T>
inline std::string V3D2string(const SklVector3D<T>* fld, const bool vvc = false)
{
	const unsigned int* _sz = fld->_GetSize();
	const unsigned int* sz  = (vvc)? _sz:fld->GetSize();
	const unsigned int  vc  = (vvc)? fld->GetVCellSize(): 0;

	//
	std::stringstream os;
	os << sz[0] << ',' << sz[1] << ',' << sz[2] << ":" << vc << std::endl;

	for (unsigned int k = 0; k < sz[2] + 2 * vc; k++) {
		os << "k = " << k << endl;
		for (unsigned int j = 0; j < sz[1] + 2 * vc; j++) {
			for (unsigned int l = 0; l < 3; l++) {
				for (unsigned int i = 0; i < sz[0] + 2 * vc; i++) {
					size_t idx = SklUtil::getCindexV3D(sz,vc,i,j,k,l);
					T v = (*fld)[idx];
					os << UtilStrFormat(" %+1.7e", v);
				}
				os << std::endl;
			}
			os << std::endl;
		}
		os << std::endl;
	}

	return os.str();
};


class IdRange {
	int _begin;
	int _end;

public:
	IdRange();
	IdRange(const std::string& tuplestr);

	int  begin() const;
	int  end()   const;

	bool isWithin(const int id) const;

	static bool isTuple(const std::string& str)
	{
		const size_t langle = str.find_first_of('(');
		const size_t rangle = str.find_last_of (')');
		const size_t fsep   = str.find_first_of(',');
		const size_t lsep   = str.find_last_of (',');

		return (langle < fsep) && (fsep == lsep) && (lsep < rangle);
	}

};

template <class T> inline
int Clipping(SklScalar3D<T>* dst, const SklScalar3D<T>* src,
				const size_t* dhead, const size_t* shead) {

	const size_t* dsize = dst->GetSize();
	const size_t* ssize = src->GetSize();

	const int dtail[] = {
			dhead[0] + dsize[0] - 1,
			dhead[1] + dsize[1] - 1,
			dhead[2] + dsize[2] - 1,
		};
	const int stail[] = {
			shead[0] + ssize[0] - 1,
			shead[1] + ssize[1] - 1,
			shead[2] + ssize[2] - 1,
		};

	// offset
	const int dI = static_cast<int>(dhead[0]) - static_cast<int>(shead[0]);
	const int dJ = static_cast<int>(dhead[1]) - static_cast<int>(shead[1]);
	const int dK = static_cast<int>(dhead[2]) - static_cast<int>(shead[2]);

	// transport range --- over wrapped volume --- on dist coord
	const int Ro[] = { // origin of range
			((dI > 0)? dI: 0),
			((dJ > 0)? dJ: 0),
			((dK > 0)? dK: 0),
		};
	const int Re[] = { // end of range
			((dtail[0] < stail[0])? dtail[0]: stail[0]) - dhead[0],
			((dtail[1] < stail[1])? dtail[1]: stail[1]) - dhead[1],
			((dtail[2] < stail[2])? dtail[2]: stail[2]) - dhead[2],
		};

	// is over wrapping?
#if 0
	DB(CERR(0) << shead[0] << ',' << shead[1] << ',' << shead[2] << endl);
	DB(CERR(0) << dhead[0] << ',' << dhead[1] << ',' << dhead[2] << endl);
	DB(CERR(0) << stail[0] << ',' << stail[1] << ',' << stail[2] << endl);
	DB(CERR(0) << dtail[0] << ',' << dtail[1] << ',' << dtail[2] << endl);
#endif
	bool wrapping = (shead[0] <= dtail[0]) &&
					(shead[1] <= dtail[1]) &&
					(shead[2] <= dtail[2]) &&
					(dhead[0] <= stail[0]) &&
					(dhead[1] <= stail[1]) &&
					(dhead[2] <= stail[2]);
	if (!wrapping) { return 0; } // no wrapped...

	// field array size
	const int dvc = dst->GetVCellSize();
	const int svc = src->GetVCellSize();

	for (int dk = -dvc; dk < Re[2] + dvc; dk++) {
	for (int dj = -dvc; dj < Re[1] + dvc; dj++) {
	for (int di = -dvc; di < Re[0] + dvc; di++) {
		// convert base coord to patch coord
		const int si = di + dI;
		const int sj = dj + dJ;
		const int sk = dk + dK;

		dst->val(di,dj,dk) = src->val(si,sj,sk);
	}}}
#if 0
	DB(CERR(0) << S3D2string(src, true) << endl);
	DB(CERR(0) << S3D2string(dst, true) << endl);
#endif
	return 1;
};

template <class T> inline
int Clipping(SklScalar4D<T>* dst, const SklScalar3D<T>* src,
				const size_t N, const size_t* dhead, const size_t* shead)
{
	SklScalar3D<T>* dst3d = dst->ExtractData(N);
	Clipping(dst3d, src, dhead, shead);
	dst->SetData(N, dst3d);
#if 0
	DB(CERR(0) << S3D2string(dst3d, true) << endl);
#endif
	delete dst3d;

	return 1;
};

#endif //__UTIL_H__

