/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */

#include "AdvectionSolv.h"

//////////////////////////////////////////////////
// Allocate Voxel
//
int AdvectionSolv::AllocateVoxel( size_t& total_mem,
				  const size_t mats, const size_t eqns)
{
#ifndef _NO_MESSAGE
	MESSAGE("Allocation for Advection");
#else
	MESSAGE("Allocation for Advection", 0);
#endif

	total_mem = 0;

	const int& gid = cfg.grpP.gid;

	size_t n = 0;
	const unsigned long ni = cfg.Pdiv.size[0];
	const unsigned long nj = cfg.Pdiv.size[1];
	const unsigned long nk = cfg.Pdiv.size[2];

	const unsigned long size_mat[4] = {ni, nj, nk, mats};
	const unsigned long size_eqn[4] = {ni, nj, nk, eqns};

	// Scalar4D
	if( !(dc_ad = dynamic_cast<SklScalar4D<SKL_REAL>*>(
			cfg.solver->SklAllocateArray(para_mng,
					"ad", SKL_CLASS_SCALAR4D,
					SKL_ARRAY_DTYPE_REAL,
					size_eqn, vc, gid))) ) return -1;

	if( !(dc_P0 = dynamic_cast<SklScalar4D<SKL_REAL>*>(
			cfg.solver->SklAllocateArray(para_mng,
					"P0", SKL_CLASS_SCALAR4D,
					SKL_ARRAY_DTYPE_REAL,
					size_eqn, vc, gid))) ) return -1;

	if( !(dc_uVel = dynamic_cast<SklScalar4D<SKL_REAL>*>(
			cfg.solver->SklAllocateArray(para_mng,
					"uVel", SKL_CLASS_SCALAR4D,
					SKL_ARRAY_DTYPE_REAL,
					size_mat, vc, gid))) ) return -1;

	if( !(dc_vVel = dynamic_cast<SklScalar4D<SKL_REAL>*>(
			cfg.solver->SklAllocateArray(para_mng,
					"vVel", SKL_CLASS_SCALAR4D,
					SKL_ARRAY_DTYPE_REAL,
					size_mat, vc, gid))) ) return -1;

	if( !(dc_wVel = dynamic_cast<SklScalar4D<SKL_REAL>*>(
			cfg.solver->SklAllocateArray(para_mng,
					"wVel", SKL_CLASS_SCALAR4D,
					SKL_ARRAY_DTYPE_REAL,
					size_mat, vc, gid))) ) return -1;

	if( !(ad = dc_ad->GetData()) ) return -1;
	if( !(P0 = dc_P0->GetData()) ) return -1;
	if( !(uVel = dc_uVel->GetData()) ) return -1;
	if( !(vVel = dc_vVel->GetData()) ) return -1;
	if( !(wVel = dc_wVel->GetData()) ) return -1;

	n = (ni + 2 * vc) * (nj + 2 * vc) * (nk + 2 * vc) * eqns;
	if( !cfg.solver->SklInitializeSKL_REAL(ad, 0.0, n) ) return -1;
	if( !cfg.solver->SklInitializeSKL_REAL(P0, 0.0, n) ) return -1;
	total_mem += n * sizeof(SKL_REAL) * 2;

	n = (ni + 2 * vc) * (nj + 2 * vc) * (nk + 2 * vc) * mats;
	if( !cfg.solver->SklInitializeSKL_REAL(uVel, 0.0, n) ) return -1;
	if( !cfg.solver->SklInitializeSKL_REAL(vVel, 0.0, n) ) return -1;
	if( !cfg.solver->SklInitializeSKL_REAL(wVel, 0.0, n) ) return -1;
	total_mem += n * sizeof(SKL_REAL) * 3;

	return 1;
}
