/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */

#include "AdvectionSolv.h"
#include <cmath>

//////////////////////////////////////////////////
// Load Velocity
//
bool AdvectionSolv::LoadVel()
{
	// total time
	SKL_REAL ttime = cfg.solver->SklGetTotalTime();
	SKL_REAL dt = RicsGetDeltaT();

	const int gid = cfg.grpP.gid;

	const int vc = VC;
	const int I  = cfg.Pdiv.size[0] + 1;
	const int J  = cfg.Pdiv.size[1] + 1;
	const int K  = cfg.Pdiv.size[2] + 1;

	const size_t mats = cfg.MatCount;

	for( int m = 0; m < mats; m++ ){
	for( int t = 0; t < velFileSize; t++ ){
		const Material* mat  = UtilGetMaterialId( VelFileTbl[t]->matName );

		if( m != mat->id ) { continue; }				// Material Check
		if( fabs(ttime - VelFileTbl[t]->time) > (dt / 10) ) { continue; }	// Time Check

		const char* attr = VelFileTbl[t]->attr.c_str();

		// const char* attr = (cfg.MatTbl[m]->name + string(":Vel")).c_str();

#ifdef SPHERE175
		SklVoxDataSet* dSet = cfg.solver->LoadFile(para_mng, attr, vc, gid);
#else
		SklVoxDataSet* dSet = cfg.solver->LoadFile(attr, vc, gid);
#endif

		if( !dSet ) { return false; }

		// if( dSet->GetClassType() != SklVoxDataSet::SKL_CLASS_VECTOR3DEX ){
		//	MSG_ERROR("");
		//	return false;
		// }

		if( dSet->GetDataType() == SklVoxDataSet::TYPE_SINGLE ){
			SklVector3DEx<float>* in = (dSet)?
					dynamic_cast<SklVector3DEx<float>*>(dSet->GetData()):
					NULL;

			for( size_t k = 0; k < K; k++ ){
			for( size_t j = 0; j < J; j++ ){
			for( size_t i = 0; i < I; i++ ){
				dc_uVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(0,i,j,k));
				dc_vVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(1,i,j,k));
				dc_wVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(2,i,j,k));
			}}}
		}
		else if( dSet->GetDataType() == SklVoxDataSet::TYPE_DOUBLE ){
			SklVector3DEx<double>* in = (dSet)?
					dynamic_cast<SklVector3DEx<double>*>(dSet->GetData()):
					NULL;

			for( size_t k = 0; k < K; k++ ){
			for( size_t j = 0; j < J; j++ ){
			for( size_t i = 0; i < I; i++ ){
				dc_uVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(0,i,j,k));
				dc_vVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(1,i,j,k));
				dc_wVel->val(i,j,k,m) = static_cast<SKL_REAL>(in->val(2,i,j,k));
			}}}
		}
		else{
			delete dSet;
			MSG_ERROR(string(attr) + " data type is unknown");
		}

		delete dSet;

#ifndef _NO_MESSAGE
		MESSAGE(UtilStrFormat("Sph File is loaded as attr=%s", attr));
#else
		MESSAGE(UtilStrFormat("Sph File is loaded as attr=%s", attr),0);
#endif //_NO_MESSAGE
	}}

	return true;
}

