//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __ADVECTION_SOLV_H__
#define __ADVECTION_SOLV_H__

#include "AdvectionDefine.h"
#include "FortranFuncAdvection.h"

class AdvectionSolv
{
protected:

// member variables
	const CfgInfo& cfg;
	const int vc;

	double clock;
	double comm_clk;
	double wait_clk;

	bool enableAdvection;

	size_t total_mem;

	// for parallel
	SklParaComponent* para_cmp;
	SklParaManager*   para_mng;

	int m_nID[6];

	// Velocity file Table
	vector<const VelFile*> VelFileTbl;
	int velFileSize;

	//////////////////////////////////////////////////
	// data class
	const S4D_REAL *phi0;
	const S4D_REAL *Vf;
	const S4D_REAL *Vfi;
	const S4D_REAL *Ax;
	const S4D_REAL *Ay;
	const S4D_REAL *Az;
	const S4D_INT *BCx;
	const S4D_INT *BCy;
	const S4D_INT *BCz;


	SklScalar4D<SKL_REAL> *dc_ad;
	SklScalar4D<SKL_REAL> *dc_P0;

	SklScalar4D<SKL_REAL> *dc_uVel, *dc_vVel, *dc_wVel;

	//////////////////////////////////////////////////
	// data class ptr
	SKL_REAL *ad;
	SKL_REAL *P0;

	SKL_REAL *uVel, *vVel, *wVel;

public:

// member function
	//////////////////////////////////////////////////
	// Constractor & Destractor (AdvectionSolv.C)
	AdvectionSolv(const CfgInfo& _cfg);
	virtual ~AdvectionSolv();

	// Getter method : enableAdvection
	bool GetEnableAdvection();

	//////////////////////////////////////////////////
	// AdvectionSolv Initialize (AdvectionSolv.C)
	int AdvectionInitialize(const CfgElem* simElem, const CfgElem* advectionElem,
				const S4D_REAL* p0, const S4D_REAL* vf, const S4D_REAL* vfi,
				const S4D_REAL* ax, const S4D_REAL* ay, const S4D_REAL* az,
				const S4D_INT* obx, const S4D_INT* oby, const S4D_INT* obz);

	// AdvectionSolv Loop (AdvectionSolv.C)
	int AdvectionLoop(const unsigned int step);

	// AdvectionSolv Post (AdvectionSolv.C)
	int AdvectionPost();


	//////////////////////////////////////////////////
	// Check config (AdvectionSolv.C)
	bool chkCfg();

	// Parse Velocity file Table (AdvectionSolv.C)
	bool ParseVelocityTable();

	// Calculation Boundary Cell (AdvectionSolv.C)
	SKL_REAL CalcBndCell(const int bc, const SKL_REAL c,
			     const SKL_REAL dx, const int face);

	// Set Boudary Cell (AdvectionSolv.C)
	bool SetBndCell();

	// Exptim Advection (AdvectionSolv.C)
	int ExptimAdv(const SKL_REAL dt, SklScalar4D<SKL_REAL> *RICS_P1);

	// Copy P0 : RICS solver main => Advection solver (AdvectionSolv.C)
	// bool CopyP0(SklScalar4D<SKL_REAL> *RICS_P0);
	bool CopyP0();


	//////////////////////////////////////////////////
	// Allocate (AdvectionAllocate.C)
	int AllocateVoxel( size_t& total_mem, const size_t mats, const size_t eqns);


	//////////////////////////////////////////////////
	// Load Velocity (AdvectionLoadfile.C)
	bool LoadVel();

	//////////////////////////////////////////////////
	// Clock
	double GetClock()   const { return clock;    }
	double GetCommClk() const { return comm_clk; }
	double GetWaitClk() const { return wait_clk; }

	//////////////////////////////////////////////////
	// Memory
	size_t GetMemory() const { return total_mem; }
};

#endif /* __ADVECTION_SOLV_H__ */
