//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __DIFFUS_SOLV_H__
#define __DIFFUS_SOLV_H__

#include <Foundation/datatype.h>

class DiffusSolv {
	const CfgInfo& cfg;
	const int vc;

	bool enableDiffusion;
	int diffTypeFlag;

	double clock;
	double comm_clk;
	double wait_clk;  // 2011.05.11

	int nID[6];

	// data class
	const S4D_REAL *P0;  //
	const S4D_REAL *Ax;  //
	const S4D_REAL *Ay;  //
	const S4D_REAL *Az;  //
	S4D_REAL *Cx;        //
	S4D_REAL *Cy;        //
	S4D_REAL *Cz;        //
	S4D_REAL *FluxX;     // destination
	S4D_REAL *FluxY;     // destination
	S4D_REAL *FluxZ;     // destination

	const S4D_INT *BCx;
	const S4D_INT *BCy;
	const S4D_INT *BCz;

	S4D_REAL *MatVfInv;
	S4D_REAL *Rhs;
	S4D_REAL *Rhso;

	S4D_REAL *p0;
	S4D_REAL *cx, *cy,*cz;

	int Allocate();
	int SetBndCellCenterCoef();

	int CalcCoef();
	int SetBndCell();
	SKL_REAL CalcBndCell(
		const int bc, const SKL_REAL c,
		const SKL_REAL dx, const int face);

public:
#ifdef SPHERE175
	//ParaComponent, ParaManager	2009.12.07
	SklParaComponent *para_cmp;
	SklParaManager *para_mng;
#endif

	DiffusSolv(const CfgInfo& _cfg);
	int DiffusionInitialize(
				const CfgElem* sim,
				S4D_REAL* FluxX, S4D_REAL* FluxY, S4D_REAL* FluxZ,
				const S4D_REAL* p0,
				S4D_REAL* dcX, S4D_REAL* dcY, S4D_REAL* dcZ,
				const S4D_REAL* baX, const S4D_REAL* baY, const S4D_REAL* baZ,
				const S4D_INT*  obX, const S4D_INT*  obY, const S4D_INT*  obZ,
				S4D_REAL* matVfInv, S4D_REAL* rhs, S4D_REAL* rhso);
	int DiffusionLoop(const SKL_REAL time);
	int DiffusionLoopEuler(const SKL_REAL time);
	int DiffusionLoopRK(const SKL_REAL time);
	int DiffusionPost();
	int chkCfg();

	SKL_REAL CourantCond();

	double GetClock()   const { return clock;    }
	double GetCommClk() const { return comm_clk; }
	double GetWaitClk() const { return wait_clk; }  // 2011.05.11
};

#endif // __DIFFUS_SOLV_H__

