//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _SKL_FORTRAN_FUNC_DiffEq_H_
#define _SKL_FORTRAN_FUNC_DiffEq_H_
#include "Skl.h"

extern "C" {
/* USER WRITE
   if using fortran subroutine,
   Please write fortran subroutine here.
   # The function described here belongs to the name scope of C.
   # Therefore, The identifier of Solver is recommended to be added to
   # the function name to distinguish from other functions.
*/

	void
	differential_(int* Flag, const SKL_REAL* P,
			const SKL_REAL* cx, const SKL_REAL* cy, const SKL_REAL* cz,
			const SKL_REAL* ax, const SKL_REAL* ay, const SKL_REAL* az,
			SKL_REAL* diffx, SKL_REAL* diffy, SKL_REAL* diffz,
			const int* imax, const int* jmax, const int* kmax, const int* vc,
			const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi);

	void
	flux_(SKL_REAL* Dx, SKL_REAL* Dy, SKL_REAL* Dz,
			const SKL_REAL* diffx, const SKL_REAL* diffy, const SKL_REAL* diffz,
			const int* imax, const int* jmax, const int* kmax, const int* vc);

	void
	calccoef_(SKL_REAL* cx, SKL_REAL* cy, SKL_REAL* cz,
			const SKL_REAL* _Cx, const SKL_REAL* _Cy, const SKL_REAL* _Cz,
			const int* nID,  const int* m, const int* mols,
			const int* imax, const int* jmax, const int* kmax, const int* vc);

	void
	setbndcoef_(SKL_REAL* _Cx, SKL_REAL* _Cy, SKL_REAL* _Cz,
			const int* nID,  const int* m, const int* mols,
			const int* imax, const int* jmax, const int* kmax, const int* vc);

	void
	calcrhs_(SKL_REAL* rr, const SKL_REAL* vfi,
			SKL_REAL* dfx, SKL_REAL* dfy, SKL_REAL* dfz,
			const int* imax, const int* jmax, const int* kmax, const int* vc,
			const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi);

	void
	exptimrk_diff_(const SKL_REAL* p, SKL_REAL* rr, const SKL_REAL* dt, SKL_REAL* coef,
			const int* imax, const int* jmax, const int* kmax, const int* vc);

	//void
	//debugwrite_(int* rstep,
	//		const int* imax, const int* jmax, const int* kmax,
	//		const SKL_REAL* rrtmp, const int* vc, const SKL_REAL* dt);
}

#endif // _SKL_FORTRAN_FUNC_DiffEq_H_

