//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _SKL_FORTRAN_FUNC_MEMBEq_H_
#define _SKL_FORTRAN_FUNC_MEMBEq_H_
#include "Skl.h"

extern "C" {
/* USER WRITE
   if using fortran subroutine,
   Please write fortran subroutine here.
   # The function described here belongs to the name scope of C.
   # Therefore, The identifier of Solver is recommended to be added to
   # the function name to distinguish from other functions.
*/

	void
	calcfluxm_(SKL_REAL* fgm,
			const SKL_REAL* f1, const SKL_REAL* f2, const SKL_REAL* f3,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

	void
	calcepsilon_(SKL_REAL* eg, const SKL_REAL* sm, const SKL_REAL* fgm,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

	void
	calcfluxpene_(SKL_REAL* fgmn,
			const SKL_REAL* p0gm, const SKL_REAL* p0gn,
			const SKL_REAL* kpene,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc,
			int* flag, int* coef);

	void
	calcfluxcha_(SKL_REAL* fgmn, const SKL_REAL* R,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0gn,
			const SKL_REAL* Evm, const SKL_REAL* Evn,
			const SKL_REAL* KCha, const SKL_REAL*KeCha,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz, 
			const int* imax, const int* jmax, const int* kmax,
			const int* vc,
			int* flag, int* coef);

	void
	calcfluxpum_(SKL_REAL* fgmn, SKL_REAL* fgnm, const SKL_REAL* R,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0hm,
			const SKL_REAL* Vmax, const SKL_REAL* Km,
			const SKL_REAL* nxm, const SKL_REAL* nym, const SKL_REAL* nzm, 
			const SKL_REAL* nxn, const SKL_REAL* nyn, const SKL_REAL* nzn, 
			const int* imax, const int* jmax, const int* kmax,
			const int* vc,
			int* flag, int* coef);

	void
	calcfluxpumtr_(SKL_REAL* qhm, const SKL_REAL* TR,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0hm,
			const SKL_REAL* Vmax, const SKL_REAL* Km,
			const SKL_REAL* Sm,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz, 
			const int* imax, const int* jmax, const int* kmax,
			const int* vc, int* coef);

	SKL_REAL
	fluxpenevoxel_(
			const SKL_REAL* p0gm, const SKL_REAL* p0gn,
			const SKL_REAL* kpene,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz,
			const int* i, const int* j, const int* k,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

	SKL_REAL
	fluxchannelvoxel_(const SKL_REAL* R,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0gn,
			const SKL_REAL* Evm, const SKL_REAL* Evn,
			const SKL_REAL* KCha, const SKL_REAL*KeCha,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz, 
			const int* i, const int* j, const int* k, 
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

	SKL_REAL
	fluxpumpvoxel_(const SKL_REAL* R,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0hm,
			const SKL_REAL* Vmax, const SKL_REAL* Km,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz, 
			const int* i, const int* j, const int* k,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

	SKL_REAL
	fluxpumptrvoxel_(const SKL_REAL* TR,
			const SKL_REAL* SW, const SKL_REAL* fuzzy, const SKL_REAL* Den,
			const SKL_REAL* p0gm, const SKL_REAL* p0hm,
			const SKL_REAL* Vmax, const SKL_REAL* Km,
			const SKL_REAL* nx, const SKL_REAL* ny, const SKL_REAL* nz, 
			const int* i, const int* j, const int* k,
			const int* imax, const int* jmax, const int* kmax,
			const int* vc);

}
#endif // _SKL_FORTRAN_FUNC_MEMBEq_H_

