//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __MEMBRANE_SOLV_H__
#define __MEMBRANE_SOLV_H__

#include <list>
#include <Foundation/datatype.h>
#include "SwController.h"

class MembraneSolv {

	CfgInfo& cfg;
	const int vc;

	bool bM;

	bool bPe;
	bool bCh;
	bool bPm;
	bool bCS;
	bool bPS;
	bool bCA;
	bool bPA;
	bool bGp;

	double clock;
	double comm_clk;
	double wait_clk; // 2011.05.11

	// each source terms
	S4D_REAL* MemTrm;
	S4D_REAL* Flux; // Flux'(g,m)

	// shared
	const S4D_REAL* P0;
	const S4D_REAL* Ev;
	const S4D_REAL* Vf;
	const S4D_REAL* Vfi;
	const S4D_REAL* Ax;
	const S4D_REAL* Ay;
	const S4D_REAL* Az;
	const S4D_REAL* Nx;
	const S4D_REAL* Ny;
	const S4D_REAL* Nz;
	const S4D_REAL* Sm;
	const S4D_REAL* SmC;
	const S4D_REAL* SmX;
	const S4D_REAL* SmY;
	const S4D_REAL* SmZ;

	// shared
	S4D_REAL* Smn;

	// shared
	S4D_REAL* FluxM;   // Flux(g,m)
	S4D_REAL* FluxMN;  // Flux(g,m,n)
	S3D_REAL* Epsilon; // epsilon(g)
	S4D_REAL* Q;       // Q(g,m): source term Q = S.f for exception membrane and energy moleculars

	// penetration
	S4D_REAL* Kpene;

	// channel
	S4D_REAL* SWcha;
	S4D_REAL* DenCha;
	S4D_REAL* FuzzyCha;
	S4D_REAL* KCha;
	S4D_REAL* KeCha;

	// pump
	S4D_REAL* SWpum;
	S4D_REAL* DenPum;
	S4D_REAL* FuzzyPum;
	S4D_REAL* VmaxPumAtp;
	S4D_REAL* KmPumAtp;
	S4D_REAL* VmaxPumGtp;
	S4D_REAL* KmPumGtp;

	// Symport channel
	S4D_REAL* SWchaSym;
	S4D_REAL* DenChaSym;
	S4D_REAL* FuzzyChaSym;
	S4D_REAL* KChaSym;
	S4D_REAL* KeChaSym;

	// Symport pump
	S4D_REAL* SWpumSym;
	S4D_REAL* DenPumSym;
	S4D_REAL* FuzzyPumSym;
	S4D_REAL* VmaxPumSymAtp;
	S4D_REAL* KmPumSymAtp;
	S4D_REAL* VmaxPumSymGtp;
	S4D_REAL* KmPumSymGtp;

	// Antiport channel
	S4D_REAL* SWchaAnti;
	S4D_REAL* DenChaAnti;
	S4D_REAL* FuzzyChaAnti;
	S4D_REAL* KChaAnti;
	S4D_REAL* KeChaAnti;

	// Antiport pump
	S4D_REAL* SWpumAnti;
	S4D_REAL* DenPumAnti;
	S4D_REAL* FuzzyPumAnti;
	S4D_REAL* VmaxPumAntiAtp;
	S4D_REAL* KmPumAntiAtp;
	S4D_REAL* VmaxPumAntiGtp;
	S4D_REAL* KmPumAntiGtp;

	// Gap Janction
	S4D_REAL* SWgap;
	S4D_REAL* DenGap;
	S4D_REAL* FuzzyGap;
	S4D_REAL* KGap;
	S4D_REAL* KeGap;

	struct ExceptFace {
		const Material* m;
		SKL_REAL nx,ny,nz;
		SKL_REAL S;
		int i,j,k;
	};
	typedef std::vector<ExceptFace> ExceptFaceListT;
	ExceptFaceListT ExceptFaceList;

	typedef std::map<const Transporter*, const SwController*> SwCtrlMapT;
	SwCtrlMapT SwChCtrlMap;
	SwCtrlMapT SwPmCtrlMap;
	SwCtrlMapT SwCSCtrlMap;
	SwCtrlMapT SwPSCtrlMap;
	SwCtrlMapT SwCACtrlMap;
	SwCtrlMapT SwPACtrlMap;
	SwCtrlMapT SwGpCtrlMap;

	int SwControl(const SwCtrlMapT& m, S4D_REAL* SW, int* mcoef);

	int LoadFunctionSetting();
	int LoadSwCtrlSetting(SwCtrlMapT& m,const Transporter* p,const CfgElem* e);

	int LoadPeSetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadChSetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadPmSetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadCSSetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadPSSetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadCASetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadPASetting(const MolMemb* gmn, const CfgElem* ml);
	int LoadGpSetting(const MolMemb* gmn, const CfgElem* ml);

	int Allocate();
	int SetInitialValue();
	int CalcMembPairArea();
	int construct_ExceptMembraneList();

	int RegistFields();

	int FluxChaGeneric(const ChTranser* p,
		const S4D_REAL* SW, const S4D_REAL* Fuzzy, const S4D_REAL* Den,
		const S4D_REAL* K, const S4D_REAL* Ke, const SKL_REAL Rsign,
		int* mcoef);
	int FluxPumGeneric(const PmTranser* p,
		const S4D_REAL* SW, const S4D_REAL* Fuzzy, const S4D_REAL* Den,
		const S4D_REAL* VmaxATP, const S4D_REAL* KmATP,
		const S4D_REAL* VmaxGTP, const S4D_REAL* KmGTP,
		const SKL_REAL Rsign,
		int* mcoef);

	int FluxPene(const penetrater* p,   int* mcoef);
	int FluxCha(const ChTranser* p,     int* mcoef);
	int FluxPum(const PmTranser* p,     int* mcoef);
	int FluxChaSym(const CSTranser* p,  int* mcoef);
	int FluxPumSym(const PSTranser* p,  int* mcoef);
	int FluxChaAnti(const CATranser* p, int* mcoef);
	int FluxPumAnti(const PATranser* p, int* mcoef);
	int FluxGap(const GpTranser* p,     int* mcoef);

	int GeneMembSourceTerm(const Molecular* g, const Material* m);

	int CalcFaceFlux(int* mcoef);
	int CalcFlux(const Equation* gm);
	int CalcFluxM(const Equation* gm);
	int CalcEpsilon(const Molecular* g);
	int CalcMembTrmField(const Equation* gm, int* mcoef);

	const Membarea* CheckMembArea(int* flag, const Material* mat1, const Material* mat2);

#ifdef MAKE_MODEL
	int MakeChannelDen();
#endif // MAKE_MODEL

public:
	MembraneSolv(CfgInfo& _cfg);

	int MembraneInitialize(
				const CfgElem* sim,
				S4D_REAL* memtrm, S4D_REAL* flux, S4D_REAL* q,
				const S4D_REAL* p0,
				const S4D_REAL* vf, const S4D_REAL* vfi,
				const S4D_REAL* ax, const S4D_REAL* ay, const S4D_REAL* az,
				const S4D_REAL* ev,
				const S4D_REAL* sm,
				const S4D_REAL* nmx, const S4D_REAL* nmy, const S4D_REAL* nmz,
				const S4D_REAL* smc,
				const S4D_REAL* smx, const S4D_REAL* smy, const S4D_REAL* smz);

	int Calc(const SKL_REAL dt);
	int chkCfg();
	int MembranePost();

	double GetClock()   const { return clock; }
	double GetCommClk() const { return comm_clk; }
	double GetWaitClk() const { return wait_clk; }  // 2011.05.11
};

#endif // __MEMBRANE_SOLV_H__

