//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include <cmath>
#include "SwController.h"

SKL_REAL SwCtrl::michaelis(const SKL_REAL S, const SwFuncParam& p)
{
	//   Vmax * S
	//  -----------
	//    Km + S
	return p.p.michaelis.Vmax * S / (p.p.michaelis.Km + S);
}

SKL_REAL SwCtrl::hill(const SKL_REAL S, const SwFuncParam& p)
{
	//      S**n
	//  -----------
	//   Kd + S**n
	const SKL_REAL Ln = std::pow(S,p.p.hill.n);
	return Ln / (p.p.hill.Kd + Ln);
}

SKL_REAL SwCtrl::linear(const SKL_REAL S, const SwFuncParam& p)
{
	//
	//  a * S + b
	//
	return p.p.linear.a * S + p.p.linear.b;
}

SKL_REAL SwCtrl::exp(const SKL_REAL S, const SwFuncParam& p)
{
	//
	//  A * exp(a * S)
	//
	return p.p.exp.A * std::exp(p.p.exp.a * S);
}

SKL_REAL SwCtrl::log(const SKL_REAL S, const SwFuncParam& p)
{
	//
	//  A * log(a * S)
	//
	return p.p.log.A * std::log(p.p.log.a * S);
}

SKL_REAL SwCtrl::log10(const SKL_REAL S, const SwFuncParam& p)
{
	//
	//  A * log10(a * S)
	//
	return p.p.log10.A * std::log10(p.p.log10.a * S);
}

SKL_REAL SwCtrl::sum(const SKL_REAL x, const SKL_REAL y) { return x + y; }
SKL_REAL SwCtrl::mul(const SKL_REAL x, const SKL_REAL y) { return x * y; }
SKL_REAL SwCtrl::min(const SKL_REAL x, const SKL_REAL y) { return std::min(x,y); }
SKL_REAL SwCtrl::max(const SKL_REAL x, const SKL_REAL y) { return std::max(x,y); }

