//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __SW_CONTROLLER_H__
#define __SW_CONTROLLER_H__

#include <SklSolverBase.h>
#include <vector>

#include <Foundation/element.h>

struct SwFuncParam {
	const Equation* gm;
	union paramT {
		struct michaelisT { SKL_REAL Vmax,Km; } michaelis;
		struct HillT      { SKL_REAL Kd,n; }    hill;
		struct Linear     { SKL_REAL a,b; }     linear;
		struct ExpT       { SKL_REAL A,a; }     exp;
		struct LogT       { SKL_REAL A,a; }     log,log10;
	} p;
};

// functions
namespace SwCtrl {
	SKL_REAL michaelis(const SKL_REAL S, const SwFuncParam& p);
	SKL_REAL hill(const SKL_REAL S, const SwFuncParam& p);
	SKL_REAL linear(const SKL_REAL S, const SwFuncParam& p);
	SKL_REAL exp(const SKL_REAL S, const SwFuncParam& p);
	SKL_REAL log(const SKL_REAL S, const SwFuncParam& p);
	SKL_REAL log10(const SKL_REAL S, const SwFuncParam& p);

	SKL_REAL sum(const SKL_REAL x, const SKL_REAL y);
	SKL_REAL mul(const SKL_REAL x, const SKL_REAL y);
	SKL_REAL min(const SKL_REAL x, const SKL_REAL y);
	SKL_REAL max(const SKL_REAL x, const SKL_REAL y);
};

//
struct SwController {
	enum ASSOCIATIVE { NONE = 0,SUM, MUL, MIN, MAX };
	ASSOCIATIVE assoc;

	vector<SwFuncParam> params;
	vector<void*> funcs;
};


#endif // __SW_CONTROLLER_H__

