//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include <iostream>
#include <fstream>
#include <map>

#include <cfloat>
#include <cmath>
#include <cstring>

#include <SklUtil.h>

#ifndef _WITHOUT_ECELL
#include <libecs/libecs.hpp>
#include <libecs/DataPoint.hpp>
#include <libecs/DataPointVector.hpp>
#endif // _WITHOUT_ECELL

#include <Foundation/ecellif/ecellif.h>
#include <Foundation/ecellif/emcsim.h>
#include <Foundation/ecellif/EmlModel.h>
#include <Foundation/datatype.h>
#include <Foundation/error.h>
#include <Foundation/util.h>
#include <Foundation/const.h>

#include "MetaboSolv.h"

MetaboSolv::MetaboSolv(const CfgInfo& _cfg)
: cfg(_cfg), vc(0)
{
	clock    = double(0.0);
	comm_clk = double(0.0);
	wait_clk = double(0.0);  // 2011.05.11
	enableMetabo = false;
}

int MetaboSolv::_split(const size_t I, const size_t J, const size_t K)
{

	const SKL_REAL Vol  = cfg.pitch[0] * cfg.pitch[1] * cfg.pitch[2];
	const SKL_REAL iVol = SKL_REAL(1.0) / Vol;

	const int gid = cfg.grpE.gid;
	S3D_REAL h(I,J,K,VC);

	UtilClearArray(fm,SKL_REAL(1.0));

	for (size_t mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		const int matid = mtb->mat->id;

		if (!cfg.EmatAss[mtb->mat->id])  { continue; }

		DB3(MESSAGE("_split" + mtb->name));
		DB5(MESSAGE(string("volume ratio of medium ") + mtb->mat->name));
		DB5(MESSAGE(S4D2string(Vfm,mtbid)));

		if (mtb->onMemb) {
			const string name     = mtb->name;
			const SKL_REAL height = mtb->height;
			UtilLoadSphOrEmmToS3D<SKL_REAL>(&h,name,height,gid);

			if( cfg.MembareaCount == 0 ){
			// No input Membarea file
				for (size_t k = 0; k < K; k++) {
				for (size_t j = 0; j < J; j++) {
				for (size_t i = 0; i < I; i++) {
					// const SKL_REAL fmt = h.val(i,j,k)
					//				* Sm->val(i,j,k,matid)
					//				* iVol * Vfim->val(i,j,k,matid);

					/*
						if( h.val(i,j,k) < SKL_REAL(0.0) ) { flag = 0; }
						else                               { flag = 1; }

						fmt = (Sm * h * iVol * Vfim) * flag + (1.0 - flag);
					*/
					// SKL_REAL fmt = SKL_REAL(0.0);
					// if( h.val(i,j,k) != SKL_REAL(0.0) ){
					//	fmt =
					//		(Sm->val(i,j,k,matid) * h.val(i,j,k)
					//		+ Sm->val(i,j,k,matid) * abs(h.val(i,j,k)))
					//		* (SKL_REAL(0.5) * iVol * Vfim->val(i,j,k,matid))
					//		+ (h.val(i,j,k) - abs(h.val(i,j,k))) / (SKL_REAL(2.0) * h.val(i,j,k));
					// }

					SKL_REAL fmt = SKL_REAL(0.0);
					if( (Sm->val(i,j,k,matid) == SKL_REAL(0.0)) || (Vfm->val(i,j,k,matid) == SKL_REAL(0.0)) ){
						fmt = SKL_REAL(0.0);
					}
					else if( h.val(i,j,k) < SKL_REAL(0.0) ){
						fmt = SKL_REAL(1.0);
					}
					else{
						fmt = h.val(i,j,k)
							* Sm->val(i,j,k,matid)
							* iVol * Vfim->val(i,j,k,matid);
					}

					fm->val(i,j,k,mtbid) = 
						(fmt < SKL_REAL(1.0))? fmt: SKL_REAL(1.0);
				}}}
			}
			else{
			// Read input MembArea file
				vector<int> MembId;
				if( mtb->contacted ){
					const Membarea* area = UtilGetMembareaId( mtb->mat, mtb->contacted );
					if( area ){
						MembId.push_back( area->id );
					}
				}
				else{
					for( int mi = 0; mi < cfg.MatCount; mi++ ){
						const Membarea* area = UtilGetMembareaId( mtb->mat, cfg.MatTbl[mi] );
						if( area ){
							MembId.push_back( area->id );
						}
					}
				}

				const size_t areaNum = 7;
				const S4D_REAL* pMembArea[areaNum];
				pMembArea[0] = SmC;
				pMembArea[1] = SmX;
				pMembArea[2] = SmY;
				pMembArea[3] = SmZ;
				pMembArea[4] = SmX;
				pMembArea[5] = SmY;
				pMembArea[6] = SmZ;

				int mcoef[areaNum][3];
				for( size_t j = 0; j < 3; j++ ){
				for( size_t i = 0; i < areaNum; i++ ){
					mcoef[i][j] = 0;
				}}
				mcoef[4][0] = 1;
				mcoef[5][1] = 1;
				mcoef[6][2] = 1;

				SKL_REAL fmtTmp;
				S3D_REAL fmt(I,J,K,VC);
				UtilClearArray(&fmt,SKL_REAL(0.0));

				for( vector<int>::iterator it = MembId.begin(); it != MembId.end(); it++ ){
				for( size_t k = 0; k < K; k++ ){
				for( size_t j = 0; j < J; j++ ){
				for( size_t i = 0; i < I; i++ ){
					SKL_REAL SmSum = SKL_REAL(0.0);
					for( size_t p = 0; p < areaNum; p++ ){
						size_t mi = i + mcoef[p][0];
						size_t mj = j + mcoef[p][1];
						size_t mk = k + mcoef[p][2];

						SmSum += pMembArea[p]->val(mi,mj,mk,*it);
					}

					fmtTmp = SKL_REAL(0.0);
					for( size_t p = 0; p < areaNum; p++ ){
						size_t mi = i + mcoef[p][0];
						size_t mj = j + mcoef[p][1];
						size_t mk = k + mcoef[p][2];

						// fmtTmp += h.val(i,j,k)
						//		* pMembArea[p]->val(mi,mj,mk,*it)
						//		* iVol * Vfim->val(i,j,k,matid);

						/*
							if( h.val(i,j,k) < SKL_REAL(0.0) ) { flag = 0; }
							else                               { flag = 1; }

							fmtTmp = (Sm * h * iVol * Vfim) * flag + (1.0 - flag);
						*/
						// fmtTmp +=
						//	(pMembArea[p]->val(mi,mj,mk,*it) * h.val(i,j,k)
						//	+ pMembArea[p]->val(mi,mj,mk,*it) * abs(h.val(i,j,k)))
						//	* (SKL_REAL(0.5) * iVol * Vfim->val(i,j,k,matid))
						//	+ (h.val(i,j,k) - abs(h.val(i,j,k))) / (SKL_REAL(2.0) * h.val(i,j,k));

						if( (SmSum == SKL_REAL(0.0)) || (Vfm->val(i,j,k,matid) == SKL_REAL(0.0)) ){
							fmtTmp = SKL_REAL(0.0);
						}
						else if( h.val(i,j,k) < SKL_REAL(0.0) ){
							fmtTmp = SKL_REAL(1.0);
						}
						else{
							fmtTmp += h.val(i,j,k)
								* pMembArea[p]->val(mi,mj,mk,*it)
								* iVol * Vfim->val(i,j,k,matid);
						}

					}
					fmt.val(i,j,k) += fmtTmp;
				}}}}

				for( size_t k = 0; k < K; k++ ){
				for( size_t j = 0; j < J; j++ ){
				for( size_t i = 0; i < I; i++ ){
					fm->val(i,j,k,mtbid) =
						(fmt.val(i,j,k) < SKL_REAL(1.0))? fmt.val(i,j,k): SKL_REAL(1.0);
				}}}
			}
		}
	}

	for (size_t matid = 0; matid < cfg.MatCount; matid++) {

		const Material*   m    = cfg.MatTbl[matid];
		const Metabolism* mtb0 = m->MtbTbl[0];
		if (!mtb0) { continue; }

		for (size_t p = 1; p < m->MtbTbl.size(); p++) {

			const Metabolism* mtb = m->MtbTbl[p];

			for (size_t k = 0; k < K; k++) {
			for (size_t j = 0; j < J; j++) {
			for (size_t i = 0; i < I; i++) {
				fm->val(i,j,k,mtb0->id) -= fm->val(i,j,k,mtb->id);
			}}}
		}
	}

	for (size_t mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		const int matid = mtb->mat->id;

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			SimMsk->val(i,j,k,mtbid) =
				(Vfm->val(i,j,k,matid) * fm->val(i,j,k,mtbid) > SKL_REAL(0.0));
		}}}

		// mask with does contacted material have volume?
		if (mtb->contacted) {
			const int mid = mtb->contacted->id;

			if( cfg.MembareaCount == 0 ){
			// No input Membarea file
				for (size_t k = 0; k < K; k++) {
				for (size_t j = 0; j < J; j++) {
				for (size_t i = 0; i < I; i++) {
					// 2011.06.30.
					// const int ni = static_cast<int>(Nx->val(i,j,k,matid));
					// const int nj = static_cast<int>(Ny->val(i,j,k,matid));
					// const int nk = static_cast<int>(Nz->val(i,j,k,matid));

					const int ni = i + static_cast<int>(Nx->val(i,j,k,matid));
					const int nj = j + static_cast<int>(Ny->val(i,j,k,matid));
					const int nk = k + static_cast<int>(Nz->val(i,j,k,matid));

					// if contacted material has voluem, mtbid is true.
					SimMsk->val(i,j,k,mtbid) &= 
						(Vfm->val(ni,nj,nk,mid) > SKL_REAL(0.0));
				}}}
			}
			else{
			// Read input Membarea file
				const Membarea* area = UtilGetMembareaId(mtb->mat, mtb->contacted);
				if( !area ) { continue; }

				for( size_t k = 0; k < K; k++ ){
				for( size_t j = 0; j < J; j++ ){
				for( size_t i = 0; i < I; i++ ){
					bool msk = false;
					msk |= ( SmC->val(i,j,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmX->val(i,j,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmY->val(i,j,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmZ->val(i,j,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmX->val(i+1,j,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmY->val(i,j+1,k,area->id) > SKL_REAL(0.0) );
					msk |= ( SmZ->val(i,j,k+1,area->id) > SKL_REAL(0.0) );

					SimMsk->val(i,j,k,mtbid) &= msk;
				}}}
			}
		}
		
//		MESSAGE(UtilStrFormat("fm(%d)", mtbid));
//		MESSAGE(S4D2string(fm,mtbid));
	}

	return 1;
}

int MetaboSolv::_refusion(const size_t I, const size_t J, const size_t K)
{
	//
	// reinitialize fusion mask
	//

#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

	for (size_t mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		const Material* m = mtb->mat;
		const int matid = m->id;

		if (!cfg.EmatAss[matid]) { continue; }

		const SKL_REAL Vol       = cfg.cvol;
		const SKL_REAL Ls3       = cfg.Ls * cfg.Ls * cfg.Ls;
		const SKL_REAL VolLs3    = Vol * Ls3;
		const SKL_REAL VolLs3pVe = VolLs3 / mtb->Ve;

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {

			// if (!SimMsk->val(i,j,k,matid)) { continue; }
			if (!SimMsk->val(i,j,k,mtbid)) { continue; }
			if (FusMsk->val(i,j,k,matid))  { continue; }

#ifndef _WITHOUT_ECELL
			// set size to non fused area
			emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
			if (sim && sim->hasModel()) {
				const SKL_REAL Vm = fm->val(i,j,k,mtbid)
									* Vfm->val(i,j,k,matid) * VolLs3pVe;
				sim->setSize(Vm);
			}
#endif // _WITHOUT_ECELL
		}}}

		if (!mtb->reactsite) { continue; }

		if( cfg.MembareaCount == 0 ){
		// No input Membarea file
			for (size_t k = 0; k < K; k++) {
			for (size_t j = 0; j < J; j++) {
			for (size_t i = 0; i < I; i++) {

#ifndef _WITHOUT_ECELL
				emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
				if (!sim) { continue; }

				const int di = i + static_cast<int>(Nx->val(i,j,k,mtb->mat->id));
				const int dj = j + static_cast<int>(Ny->val(i,j,k,mtb->mat->id));
				const int dk = k + static_cast<int>(Nz->val(i,j,k,mtb->mat->id));

				const char* vpn = "Variable:/:BINDSITE_SIZE:Value";
				const Material* bsite = mtb->LR->mat;
				const SKL_REAL Vb = Vfm->val(di,dj,dk,bsite->id) * VolLs3pVe;

#ifdef ECELL_3_1
				sim->setEntityProperty(vpn, Vb);
#else
				sim->setEntityProperty(vpn, libecs::Polymorph(static_cast<libecs::Real>(Vb)));
#endif //ECELL_3_1
#endif // _WITHOUT_ECELL
			}}}
		}
		else{
		// Read input Membarea file
			const Membarea* area = UtilGetMembareaId(mtb->mat, mtb->LR->mat);
			if( !area ) { continue; }

			for( size_t k = 0; k < K; k++ ){
			for( size_t j = 0; j < J; j++ ){
			for( size_t i = 0; i < I; i++ ){
#ifndef _WITHOUT_ECELL
				emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
				if (!sim) { continue; }

				const char* vpn = "Variable:/:BINDSITE_SIZE:Value";
				const Material* bsite = mtb->LR->mat;

				SKL_REAL Vb = SKL_REAL(0.0);
				if( SmC->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i,j,k,bsite->id) * VolLs3pVe;
				}
				if( SmX->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i-1,j,k,bsite->id) * VolLs3pVe;
				}
				if( SmY->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i,j-1,k,bsite->id) * VolLs3pVe;
				}
				if( SmZ->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i,j,k-1,bsite->id) * VolLs3pVe;
				}
				if( SmX->val(i+1,j,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i+1,j,k,bsite->id) * VolLs3pVe;
				}
				if( SmY->val(i,j+1,k,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i,j+1,k,bsite->id) * VolLs3pVe;
				}
				if( SmZ->val(i,j,k+1,area->id) > SKL_REAL(0.0) ){
					Vb += Vfm->val(i,j,k+1,bsite->id) * VolLs3pVe;
				}

#ifdef ECELL_3_1
				sim->setEntityProperty(vpn, Vb);
#else
				sim->setEntityProperty(vpn, libecs::Polymorph(static_cast<libecs::Real>(Vb)));
#endif //ECELL_3_1
#endif // _WITHOUT_ECELL
			}}}
		}
	}

	for (size_t matid = 0; matid < cfg.MatCount; matid++) {

		const Material* m = cfg.MatTbl[matid];

		// size setting fusion area
		const bool     fus = m->fus;
		const SKL_REAL fut = m->fut;

		SKL_REAL     FusVfm   = SKL_REAL(0.0);
		unsigned int FusCells = 0;

		// calc fused area volume
		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			FusVfm   += FusMsk->val(i,j,k,matid)? Vfm->val(i,j,k,matid): 0.0;
			FusCells += FusMsk->val(i,j,k,matid)? 1: 0;
		}}}

		// share overall fusion area size inter nodes in parallel computing.
		if (cfg.Parallel && fus) {
			SKL_REAL     AreaVfm   = FusVfm;
			unsigned int AreaCells = FusCells;

#ifdef SPHERE175 // 2011.05.12
			MEASURE_WAIT_TIME(wait_clk, para_cmp->Barrier(cfg.grpE.gid))
#else
			MEASURE_WAIT_TIME(wait_clk, ParaMngr.Barrier(cfg.grpE.gid))
#endif
			BEGIN_CLOCK(comm)
#ifdef SPHERE175
			para_cmp->Allreduce	//2009.12.07
#else
			ParaMngr.Allreduce
#endif
					(&AreaVfm, &FusVfm, 1,
					SKL_ARRAY_DTYPE_REAL, SKL_SUM, cfg.grpE.gid);
#ifdef SPHERE175
			para_cmp->Allreduce	//2009.12.07
#else
			ParaMngr.Allreduce
#endif
					(&AreaCells, &FusCells, 1,
					SKL_ARRAY_DTYPE_UINT, SKL_SUM, cfg.grpE.gid);
			END_CLOCK(comm_clk,comm)
		}

		FusArea[matid].Vfm   = FusVfm;
		FusArea[matid].Vfim  = (FusVfm > SKL_REAL(0.0))?
								SKL_REAL(1.0) / FusVfm:
								SKL_REAL(0.0);
		FusArea[matid].cells = FusCells;

#ifndef _WITHOUT_ECELL
		// resize E-Cell model size
		if (fus && FusArea[matid].sim->hasModel()) {
			const SKL_REAL Vol = cfg.cvol;
			const SKL_REAL Ls3 = cfg.Ls * cfg.Ls * cfg.Ls;
			const SKL_REAL VolLs3pVe = Vol * Ls3 / m->MtbTbl[0]->Ve;
			const SKL_REAL fusvol = FusArea[matid].Vfm * VolLs3pVe;
			FusArea[matid].sim->setSize(fusvol);
		}
#endif // _WITHOUT_ECELL
	}

	return 1;
}

int MetaboSolv::_bind(const size_t I, const size_t J, const size_t K)
{
#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

	const SKL_REAL Vol = cfg.cvol;
	const SKL_REAL Ls3 = cfg.Ls * cfg.Ls * cfg.Ls;
	const SKL_REAL KnaNA = cfg.Kna * cfg.NA;
	const SKL_REAL KnaNAVolLs3 = KnaNA * Vol * Ls3;

	for (int eid = 0; eid < cfg.EqnCount; eid++) {
		const Equation*  e = cfg.EqnTbl[eid];
		const Material*  m = e->mat;

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			EVmdt->val(i,j,k,e->id) = P0->val(i,j,k,e->id)
									* Vfm->val(i,j,k,m->id)
									* KnaNAVolLs3;
		}}}
	}

#if 0
	const int ti = 2;
	const int tj = 4;
	const int tk = 2;
#endif

	// regular area
#ifndef _WITHOUT_ECELL
	for (int eid = 0; eid < cfg.EqnCount; eid++) {
	for (int mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Equation*   e   = cfg.EqnTbl[eid];
		const Molecular*  g   = e->mol;
		const Material*   m   = e->mat;
		const Metabolism* mtb = cfg.MtbTbl[mtbid];

		if (!cfg.EmatAss[m->id]) { continue; }
		if (mtb->mat != m)       { continue; }

		const string fullid = g->fullid? g->fullid: "";
		const string fullpn = fullid + ":Value";

		const bool existpn = MtbSim[mtbid]?
								MtbSim[mtbid]->isExistEntityProperty(fullpn):
								false;

		DB3(MESSAGE(string("_bind: set value to E-CELL")));
		DB5(MESSAGE(string("FullID:") + fullid + " " + e->name));
		DB5(MESSAGE(S4D2string(P0,e->id)));

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			const SKL_REAL mass = P0->val(i,j,k,e->id)
								* fm->val(i,j,k,mtbid)
								* Vfm->val(i,j,k,m->id)
								* KnaNAVolLs3;

			TRY(REAL_VALID(mass), e->name + " has over flow"); 

			emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
			if (!(sim && existpn)) { continue; }

#ifdef ECELL_3_1
			sim->setEntityProperty(fullpn, mass / mtb->Ve);
#else
			sim->setEntityProperty(fullpn, libecs::Polymorph(static_cast<libecs::Real>(mass / mtb->Ve)));
#endif //ECELL_3_1
			EVmdt->val(i,j,k,e->id) -= mass;
		}}}
	}}
#endif // _WITHOUT_ECELL

	// receptor
	for (int mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		if (!mtb->reactsite) { continue; }

		if( cfg.MembareaCount == 0 ){
		// No input Membarea file
			for (size_t k = 0; k < K; k++) {
			for (size_t j = 0; j < J; j++) {
			for (size_t i = 0; i < I; i++) {
#ifndef _WITHOUT_ECELL
				emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
				if (!sim) { continue; }

				const int di = i + static_cast<int>(Nx->val(i,j,k,mtb->mat->id));
				const int dj = j + static_cast<int>(Ny->val(i,j,k,mtb->mat->id));
				const int dk = k + static_cast<int>(Nz->val(i,j,k,mtb->mat->id));

				const char* lrpn      = "Variable:/:LR:Value";
				const Equation* LR    = mtb->LR;
				const SKL_REAL LRmass = P0->val(di,dj,dk,LR->id)
								  * Vfm->val(di,dj,dk,LR->mat->id)
								  * KnaNAVolLs3;

#ifdef ECELL_3_1
				sim->setEntityProperty(lrpn, LRmass / mtb->Ve);
#else
				sim->setEntityProperty(lrpn, libecs::Polymorph(static_cast<libecs::Real>(LRmass / mtb->Ve)));
#endif //ECELL_3_1
#endif //_WITHOUT_ECELL
			}}}
		}
		else{
		// Read input Membarea file
			const Membarea* area = UtilGetMembareaId(mtb->mat, mtb->LR->mat);
			if( !area ) { continue; }

			for (size_t k = 0; k < K; k++) {
			for (size_t j = 0; j < J; j++) {
			for (size_t i = 0; i < I; i++) {
#ifndef _WITHOUT_ECELL
				emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
				if (!sim) { continue; }

				const char* lrpn      = "Variable:/:LR:Value";
				const Equation* LR    = mtb->LR;

				SKL_REAL LRmass = SKL_REAL(0.0);
				if( SmC->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i,j,k,LR->id) * Vfm->val(i,j,k,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmX->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i-1,j,k,LR->id) * Vfm->val(i-1,j,k,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmY->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i,j-1,k,LR->id) * Vfm->val(i,j-1,k,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmZ->val(i,j,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i,j,k-1,LR->id) * Vfm->val(i,j,k-1,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmX->val(i+1,j,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i+1,j,k,LR->id) * Vfm->val(i+1,j,k,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmY->val(i,j+1,k,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i,j+1,k,LR->id) * Vfm->val(i,j+1,k,LR->mat->id) * KnaNAVolLs3;
				}
				if( SmZ->val(i,j,k+1,area->id) > SKL_REAL(0.0) ){
					LRmass += P0->val(i,j,k+1,LR->id) * Vfm->val(i,j,k+1,LR->mat->id) * KnaNAVolLs3;
				}

#ifdef ECELL_3_1
				sim->setEntityProperty(lrpn, LRmass / mtb->Ve);
#else
				sim->setEntityProperty(lrpn, libecs::Polymorph(static_cast<libecs::Real>(LRmass / mtb->Ve)));
#endif //ECELL_3_1
#endif //_WITHOUT_ECELL
			}}}

		}
	}

	// fusion area
	for (int eid = 0; eid < cfg.EqnCount; eid++) {
		const Equation*  e = cfg.EqnTbl[eid];
		const Molecular* g = e->mol;
		const Material*  m = e->mat;

		if (!cfg.EmatAss[m->id]) { continue; }

		const string fullid = g->fullid? g->fullid: "";
		const string fullpn = fullid + ":Value";

		DB3(MESSAGE(string("_bind: set value to E-CELL")));
		DB5(MESSAGE(string("FullID:") + fullid + " " + e->name));
		DB5(MESSAGE(S4D2string(P0,e->id)));

		SKL_REAL fmol = SKL_REAL(0.0); // mass in fused area
		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {

			const SKL_REAL mass = P0->val(i,j,k,e->id)
								* Vfm->val(i,j,k,m->id)
								* KnaNAVolLs3;

			if (FusMsk->val(i,j,k,m->id)) {
				fmol += mass / m->MtbTbl[0]->Ve;
				EVmdt->val(i,j,k,e->id) -= mass;
			}
		}}}

		if (cfg.Parallel && m->fus) {
			SKL_REAL areafmol;
			
#ifdef SPHERE175 // 2011.05.12
			MEASURE_WAIT_TIME(wait_clk, para_cmp->Barrier(cfg.grpE.gid))
#else
			MEASURE_WAIT_TIME(wait_clk, ParaMngr.Barrier(cfg.grpE.gid))
#endif
			BEGIN_CLOCK(comm);
#ifdef SPHERE175
			para_mng->Allreduce(&fmol, &areafmol, 1,	//2009.12.07
								SKL_ARRAY_DTYPE_REAL, SKL_SUM, cfg.grpE.gid);
#else
			ParaMngr.Allreduce(&fmol, &areafmol, 1,
								SKL_ARRAY_DTYPE_REAL, SKL_SUM, cfg.grpE.gid);
#endif
			FusArea[m->id].fmol[g->id] = areafmol;
			//
			END_CLOCK(comm_clk,comm);
		} else {
			FusArea[m->id].fmol[g->id] = fmol;
		}
		TRY(REAL_VALID(FusArea[m->id].fmol[g->id]),
				e->name + " has over flow in Fusion Area"); 

#ifndef _WITHOUT_ECELL
		if (FusArea[m->id].sim
				&& FusArea[m->id].sim->isExistEntityProperty(fullpn)) {

			const SKL_REAL fmol = FusArea[m->id].fmol[g->id];
#ifdef ECELL_3_1
			FusArea[m->id].sim->setEntityProperty(fullpn, fmol);
#else
			FusArea[m->id].sim->setEntityProperty(fullpn, libecs::Polymorph(static_cast<libecs::Real>(fmol)));
#endif //ECELL_3_1
		}
#endif // _WITHOUT_ECELL
	}

	return 1;
}

int MetaboSolv::_run(const size_t I, const size_t J, const size_t K)
{
	// The _run function try to do simulation for dt[s] on all of
	// materials and voxells.
	//
	// When catch a simulator runtime error, simulation can't continue
	// process. So, all simulation process be aborted. Runtime error is
	// an E-Cell internal error, it's unrepairable with automatically.
	
	DB2(MESSAGE("_run"));

	// run fused area
	for (int matid = 0; matid < cfg.MatCount; matid++) {

		const Material* m  = cfg.MatTbl[matid];

		if ((!cfg.Parallel)
			|| (cfg.grpE.nid == (matid % cfg.grpE.num))) {

#ifndef _WITHOUT_ECELL
			if (FusArea[matid].sim
					&& FusArea[matid].sim->hasModel()
					&& m->fus) {

				DB2(MESSAGE(m->name + " Fusioned Area"));

				const SKL_REAL dt = cfg.dt * cfg.Ts / m->MtbTbl[0]->Te;
				FusArea[matid].sim->run(dt);
				FusArea[matid].sim->stop();
			}
#endif // _WITHOUT_ECELL
		}
	}

	for (int mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		const Material*   mat = mtb->mat;

		const int matid = mat->id;

		if (!cfg.EmatAss[matid]) { continue; }

		DB2(MESSAGE(string("Run E-Cell of ") + mtb->name));

		const SKL_REAL dt = cfg.dt * cfg.Ts / mtb->Te;

		// run non fused area
		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {

			if (!SimMsk->val(i,j,k,mtbid)) { continue; }
			if (FusMsk->val(i,j,k,matid)) { continue; }

#ifndef _WITHOUT_ECELL
			emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
			if (sim && sim->hasModel()) {
				DB3(MESSAGE(string("Run E-Cell ")+ \
					mtb->name+" @"+ \
					string_cast(i)+','+string_cast(j)+','+string_cast(k)));

				sim->run(dt);
				sim->stop();
			}
#endif // _WITHOUT_ECELL
		}}}
	}

	return 1;
}


int MetaboSolv::_emitt(const size_t I, const size_t J, const size_t K)
{

#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

	// fusioned area
	for (int eid = 0; eid < cfg.EqnCount; eid++) {

		const Equation*  e = cfg.EqnTbl[eid];
		const Molecular* g = e->mol;
		const Material*  m = e->mat;

		if (!m->fus) { continue; }
		if (!cfg.EmatAss[m->id]) { continue; }

		const string fullid = g->fullid? g->fullid: "";
		const string fullpn = fullid + ":Value";

#ifndef _WITHOUT_ECELL
		const emcsim*   fsim    = FusArea[m->id].sim;
		const bool      existpn = fsim->isExistEntityProperty(fullpn);
		const unsigned int snID = (cfg.Parallel)? m->id % cfg.grpE.num: 0;

		// share result on fusion area inter nodes
		SKL_REAL  fusmol = ((cfg.grpE.nid == snID) && existpn)?
#ifdef ECELL_3_1
							fsim->getEntityProperty(fullpn).asReal():
#else
							fsim->getEntityProperty(fullpn).as<libecs::Real>():
#endif //ECELL_3_1
							FusArea[m->id].fmol[g->id];

#ifdef SPHERE175 // 2011.05.12
			MEASURE_WAIT_TIME(wait_clk, para_cmp->Barrier(cfg.grpE.gid))
#else
			MEASURE_WAIT_TIME(wait_clk, ParaMngr.Barrier(cfg.grpE.gid))
#endif
		BEGIN_CLOCK(comm)
#ifdef SPHERE175
		para_mng->Broadcast(&fusmol,1,SKL_ARRAY_DTYPE_REAL,snID,cfg.grpE.gid);
#else
		ParaMngr.Broadcast(&fusmol,1,SKL_ARRAY_DTYPE_REAL,snID,cfg.grpE.gid);
#endif
		END_CLOCK(comm_clk,comm)

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			if (FusMsk->val(i,j,k,m->id)) {
				const SKL_REAL mass = fusmol
									* FusArea[m->id].Vfim
									* m->MtbTbl[0]->Ve;
				EVmdt->val(i,j,k,e->id) += mass;
			}
		}}}
#endif // _WITHOUT_ECELL

		DB2(MESSAGE(string("_emitt values of ") + e->name));
		DB5(MESSAGE(string("FullID ") + fullid + " " + g->name));
		DB5(MESSAGE(S4D2string(EVmdt,e->id)));
	}

	// regular area
	for (int eid   = 0; eid   < cfg.EqnCount; eid++)   {
	for (int mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {
		//
		const Equation*   e   = cfg.EqnTbl[eid];
		const Metabolism* mtb = cfg.MtbTbl[mtbid];

		const Molecular*  g = e->mol;
		const Material*   m = e->mat;

		if (!cfg.EmatAss[m->id]) { continue; }
		if (m != mtb->mat) { continue; }

#ifndef _WITHOUT_ECELL
		const string fullid = g->fullid? g->fullid: "";
		const string fullpn = fullid + ":Value";

		const bool existpn = MtbSim[mtbid]->isExistEntityProperty(fullpn);

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			const emcsim* sim =
						reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
			if (sim && SimMsk->val(i,j,k,mtbid) && existpn) {
#ifdef ECELL_3_1
				const SKL_REAL mass = sim->getEntityProperty(fullpn).asReal()
#else
				const SKL_REAL mass = sim->getEntityProperty(fullpn).as<libecs::Real>()
#endif //ECELL_3_1
									* mtb->Ve;
				EVmdt->val(i,j,k,e->id) += mass;
			}
		}}}
#endif // _WITHOUT_ECELL

		DB2(MESSAGE(string("_emitt values of ") + e->name));
		DB5(MESSAGE(string("FullID ") + fullid + " " + g->name));
		DB5(MESSAGE(S4D2string(EVmdt,e->id)));
	}}

	return 1;
}

int MetaboSolv::_diff(const size_t I, const size_t J, const size_t K)
{
	const SKL_REAL Vol = cfg.cvol;
	const SKL_REAL Ls3 = cfg.Ls * cfg.Ls * cfg.Ls;
	const SKL_REAL KnaNA = cfg.Kna * cfg.NA;
	const SKL_REAL KnaNAVolLs3  = KnaNA * Vol * Ls3;
	const SKL_REAL iKnaNAVolLs3 = SKL_REAL(1.0) / KnaNAVolLs3;

	for (int eid = 0; eid < cfg.EqnCount; eid++) {

		const Equation*  e = cfg.EqnTbl[eid];
		const Molecular* g = e->mol;
		const Material*  m = e->mat;
		
		if (!cfg.EmatAss[m->id]) { continue; }
		if ((m->MtbTbl.size() == 1) && (m->MtbTbl[0] == NULL)) { continue; }

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			E->val(i,j,k,eid) =
				((EVmdt->val(i,j,k,eid) -
				 (P0->val(i,j,k,eid) * Vfm->val(i,j,k,m->id) * KnaNAVolLs3)) *
				  Vfim->val(i,j,k,m->id) * iKnaNAVolLs3) / cfg.dt;

			// E->val(i,j,k,eid) =
			//	((EVmdt->val(i,j,k,eid)
			//	  * Vfim->val(i,j,k,m->id) * iKnaNAVolLs3)
			//	 - P0->val(i,j,k,eid)) / cfg.dt;
		}}}

		DB2(MESSAGE(string("Metabo Term of ") + e->name));
		DB5(MESSAGE(S4D2string(E,e->id)));
	}

	return 1;
}

int MetaboSolv::_post(const size_t I, const size_t J, const size_t K)
{
	DB2(MESSAGE("_post"));
	return 1;
}

void MetaboSolv::_probe_output()
{

#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

	const size_t I = cfg.Ediv.size[0];
	const size_t J = cfg.Ediv.size[1];
	const size_t K = cfg.Ediv.size[2];

	const int gid = cfg.grpE.gid;
	const int nid = cfg.grpE.nid;

	const SKL_REAL btime = cfg.solver->SklGetBaseTime();

#ifndef _WITHOUT_ECELL
	for (MetaboProbeListT::iterator prb = MetaboProbeList.begin();
			prb != MetaboProbeList.end(); prb++) {

		const size_t i = prb->i;
		const size_t j = prb->j;
		const size_t k = prb->k;
		const size_t idx = S3D_INDEX(I,J,K,i,j,k,vc);
		const size_t tgtcnt = prb->obj.size();

//		size_t       ElemSize[tgtcnt];
		size_t* ElemSize = new size_t[tgtcnt];
		libecs::Real time_iter(0.0);
//		libecs::DataPointVectorIterator iter[tgtcnt];
//		libecs::Real prevalue[tgtcnt];
//		boost::shared_ptr<libecs::DataPointVector> data[tgtcnt];
#ifdef ECELL_3_1
		libecs::DataPointVectorIterator* iter = new libecs::DataPointVectorIterator[tgtcnt];
#else
		std::size_t* iter = new std::size_t[tgtcnt];
#endif //ECELL_3_1
		libecs::Real* prevalue = new libecs::Real[tgtcnt];
		boost::shared_ptr<libecs::DataPointVector>* data = new boost::shared_ptr<libecs::DataPointVector>[tgtcnt];

		if (nid == prb->nID) {
			stringstream log;
			size_t leatest = 0;

			for (size_t o = 0; o < tgtcnt; o++) {
				const MtbPrbTgtObjT& obj = prb->obj[o];
				const Metabolism* mtb = UtilGetMetabolismId(obj.first);
				if (!mtb) { continue; }
				if (!cfg.EmatAss[mtb->mat->id]) { continue; }

				const emcsim* sim =
						reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtb->id));

				data[o]     = sim->getLoggerData(obj.second);
				iter[o]     = data[o]->begin();
				ElemSize[o] = data[o]->getElementSize();
			}

			for (size_t t = 0;; t++) {
				libecs::Real ptime(-1.0);
				bool flg = false;

				// search leatest iterated time
				for (size_t o = 0; o < tgtcnt; o++) {
					if (iter[o] == data[o]->end()) { continue; }

					flg = true;
					libecs::Real time =
							(ElemSize[o] == sizeof(libecs::DataPoint))?
								data[o]->asShort(iter[o]).getTime():
								data[o]->asLong (iter[o]).getTime();
					if ((ptime < 0) || (time < ptime)) {
						leatest = o; 
						ptime   = time;
					}
				}

				if (!flg) break;
				const char sep = ',';
				const SKL_REAL time = (flg)?
										btime + ptime:
										cfg.solver->SklGetTotalTime();
				const int sp = 0;
				log << setw(sp) << time    << sep
					<< setw(sp) << prb->x  << sep
					<< setw(sp) << prb->y  << sep
					<< setw(sp) << prb->z  << sep
					<< setw(sp) << prb->_i << sep
					<< setw(sp) << prb->_j << sep
					<< setw(sp) << prb->_k << sep;

				// marge logger
				for (size_t o = 0; o < tgtcnt; o++) {
					libecs::Real time
							= (ElemSize[o] == sizeof(libecs::DataPoint))?
								data[o]->asShort(iter[o]).getTime():
								data[o]->asLong (iter[o]).getTime();
					libecs::Real value
							= (ElemSize[o] == sizeof(libecs::DataPoint))?
								data[o]->asShort(iter[o]).getValue():
								data[o]->asLong (iter[o]).getValue();

					if (time == ptime) {
						prevalue[o] = value;
						log << setw(sp) << value << sep;
						iter[o]++;
					} else {
						log << setw(sp) << prevalue[o] << sep;
					}
				}

				log << endl;
			}

			if (nid != 0) {
				// send probe log message to writer node
				const string data = log.str();
				size_t s   = data.length() + 1;
				char*  buf = const_cast<char*>(data.c_str());

				BEGIN_CLOCK(comm)

#ifdef SPHERE175
				para_mng->Send(&s, sizeof(size_t),SKL_ARRAY_DTYPE_CHAR,0,gid); // data size	//2009.12.07
				para_mng->Send(buf,s,SKL_ARRAY_DTYPE_CHAR,0,gid); // data body	//2009.12.07
#else
				ParaMngr.Send(&s, sizeof(size_t),SKL_ARRAY_DTYPE_CHAR,0,gid); // data size
				ParaMngr.Send(buf,s,SKL_ARRAY_DTYPE_CHAR,0,gid); // data body
#endif

				END_CLOCK(comm_clk,comm)
			} else {
				// direct write on writer node
				ofstream ofs(prb->fname.c_str(), std::ios::out | std::ios::app);
				ofs << log.str();
				ofs.close();
			}
		} else if (nid == 0) {
			// writer node
			BEGIN_CLOCK(comm)
			size_t s;

#ifdef SPHERE175
			para_mng->Recv(&s,sizeof(size_t),SKL_ARRAY_DTYPE_CHAR,prb->nID,gid);		//2009.12.07
#else
			ParaMngr.Recv(&s,sizeof(size_t),SKL_ARRAY_DTYPE_CHAR,prb->nID,gid);
#endif

			char* data = new char[s + 1];

#ifdef SPHERE175
			para_mng->Recv(data,s,SKL_ARRAY_DTYPE_CHAR,prb->nID,gid);	//2009.12.07
#else
			ParaMngr.Recv(data,s,SKL_ARRAY_DTYPE_CHAR,prb->nID,gid);
#endif

			END_CLOCK(comm_clk,comm)

			ofstream ofs(prb->fname.c_str(), std::ios::out | std::ios::app);
			ofs << data;
			ofs.close();

			delete[] data;
		}
		delete[] ElemSize;
		delete[] iter;
		delete[] prevalue;
		delete[] data;

	}
#endif // _WITHOUT_ECELL
}

void MetaboSolv::_output_field(const size_t I, const size_t J, const size_t K, const SKL_REAL time, const unsigned int step) {

	const int nid = cfg.grpE.nid;
	const int gid = cfg.grpE.gid;

#ifndef _WITHOUT_ECELL
	const SKL_REAL     ttime = cfg.solver->SklGetTotalTime();
	const unsigned int tstep = cfg.solver->SklGetTotalStep();
	for (OutDataMapT::iterator out = MetaboOutDataMap.begin();
			out != MetaboOutDataMap.end(); out++) {

		const string&  tgt  = out->first;
		string matname, fullpn;
		_split_tgtstr(tgt, matname, fullpn);

		const Metabolism* mtb = UtilGetMetabolismId(matname);
		if (!cfg.EmatAss[mtb->mat->id]) { continue; }

		SklVoxDataSet*  dSet = out->second;
		S3D_REAL*       aray = dynamic_cast<S3D_REAL*>(dSet->GetData());

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			// const emcsim* sim = FusMsk->val(i,j,k,mtb->id)?
			const emcsim* sim = FusMsk->val(i,j,k,mtb->mat->id)?
						FusArea[mtb->mat->id].sim:
						reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtb->id));

			aray->val(i,j,k) = sim && sim->isExistEntityProperty(fullpn)?
#ifdef ECELL_3_1
					sim->getEntityProperty(fullpn).asReal():
#else
					sim->getEntityProperty(fullpn).as<libecs::Real>():
#endif //ECELL_3_1
					aray->val(i,j,k) = SKL_REAL(0.0);
		}}}

		if (aray) {
			cfg.solver->WriteFile(dSet, tstep, ttime, gid, false);
		}
	}
#endif // _WITHOUT_ECELL
}

void MetaboSolv::_split_tgtstr(const string& tgt, string& matname, string& fullpn)
{
	const char   sep  = ':';
	const size_t fsep = tgt.find_first_of(sep);

	matname = (tgt.empty())? "": tgt.substr(0, fsep);
	fullpn  = (tgt.empty())? "": tgt.substr(fsep + 1);
}

void MetaboSolv::_construct_ecell(const size_t I, const size_t J, const size_t K)
{
	////////////////////////////////////////////////////////////////////////
	//
	// construct E-Cell instance and load EML model
	//

#ifndef _WITHOUT_ECELL
	for (size_t mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {

		const Metabolism* mtb = cfg.MtbTbl[mtbid];
		const Material*   mat = mtb->mat;

		if (!cfg.EmatAss[mat->id]) { continue; }

#ifndef _NO_MESSAGE
		MESSAGE(UtilStrFormat("Test EML '%s' of %s",mtb->eml,mtb->name.c_str()));
#else
		MESSAGE(UtilStrFormat("Test EML '%s' of %s",mtb->eml,mtb->name.c_str()),0);
#endif //_NO_MESSAGE

		EmlModel eml;
		MtbSim[mtbid] = new emcsim;

		eml.open(mtb->eml);
		MtbSim[mtbid]->testModel(eml);

		// dump stepper and entity propaties
		DB4(eml.Dump());             // at EML file
		DB4(MtbSim[mtbid]->Dump());  // at Simulator

		if (mtb->reactsite) {
			const char* msg = "Entity Property '%s' is not found";
			const char* r   = "Variable:/:LR:Value";
			const char* s   = "Variable:/:BINDSITE_SIZE:Value";
			TRY(MtbSim[mtbid]->isExistEntityProperty(r),UtilStrFormat(msg,r));
			TRY(MtbSim[mtbid]->isExistEntityProperty(s),UtilStrFormat(msg,s));
		}
#ifndef _NO_MESSAGE
		MESSAGE(UtilStrFormat("%s is ready.", mtb->eml));
		MESSAGE(UtilStrFormat("construct E-Cell of %s",mtb->name.c_str()));
#else
		MESSAGE(UtilStrFormat("%s is ready.", mtb->eml),0);
		MESSAGE(UtilStrFormat("construct E-Cell of %s",mtb->name.c_str()),0);
#endif //_NO_MESSAGE

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			EmcSim->val(i,j,k,mtbid) = NULL;
			if (SimMsk->val(i,j,k,mtbid)) {
				emcsim* sim = new emcsim(); // E-Cell simulator instance
				sim->loadModel(eml,mtb->sys);
				EmcSim->val(i,j,k,mtbid) = sim;
			}
		}}}
	}
		
	// ecell instance for fused area
	for (size_t matid = 0; matid < cfg.MatCount; matid++) {

		const Material*   m   = cfg.MatTbl[matid];
		const Metabolism* mtb = m->MtbTbl[0];

		FusArea[matid].matid = matid;
		FusArea[matid].sim   = NULL;

		if (m->fus && mtb && mtb->eml) { 
			MESSAGE(UtilStrFormat("construct E-Cell of %s for fusion",
#ifndef _NO_MESSAGE
									m->name.c_str()));
#else
									m->name.c_str()),0);
#endif //_NO_MESSAGE
			EmlModel eml;
			eml.open(mtb->eml);
			FusArea[matid].sim = new emcsim();
			FusArea[matid].sim->loadModel(eml, mtb->sys);
		}
	}
#endif // _WITHOUT_ECELL
}

void MetaboSolv::_allocate_fields(const size_t I, const size_t J, const size_t K)
{
	////////////////////////////////////////////////////////////////////////
	//
	// allocate solver local fields
	//

	EVmdt  = new S4D_REAL(I,J,K,cfg.EqnCount,vc);
	fm     = new S4D_REAL(I,J,K,cfg.MtbCount,vc);
	SimMsk = new S4D_BOOL(I,J,K,cfg.MtbCount,vc);
	FusMsk = new S4D_BOOL(I,J,K,cfg.MatCount,vc);
	EmcSim = new S4D_VOIDP(I,J,K,cfg.MtbCount,vc);

	UtilClearArray(FusMsk,false);
	UtilClearArray(EVmdt,SKL_REAL(0.0));
	UtilClearArray(fm,SKL_REAL(0.0));
	UtilClearArray<bool>(SimMsk,NULL);
	UtilClearArray<void*>(EmcSim,NULL);

#ifndef _WITHOUT_ECELL
	MtbSim.resize(cfg.MtbCount, NULL);
#endif // _WITHOUT_ECELL
}

void MetaboSolv::_init_fusion_mask(const size_t I, const size_t J, const size_t K)
{
	////////////////////////////////////////////////////////////////////////
	//
	// initialize fusion mask field
	//

	for (size_t matid = 0; matid < cfg.MatCount; matid++) {

		if (!cfg.EmatAss[matid]) { continue; }

		const Material* m   = cfg.MatTbl[matid];
		const SKL_REAL  fut = m->fut;
		const bool      fus = m->fus && (fut > SKL_REAL(0.0));

		for (size_t k = 0; k < K; k++) {
		for (size_t j = 0; j < J; j++) {
		for (size_t i = 0; i < I; i++) {
			FusMsk->val(i,j,k,matid) = fus && (Vfm->val(i,j,k,matid) >= fut);
		}}}
	}
}

void MetaboSolv::_init_fldout(const CfgElem* output, const bool file_initialize)
{

#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

#ifndef _WITHOUT_ECELL
	////////////////////////////////////////////////////////////////////////
	//
	// field output setting
	//
	const size_t I = cfg.Ediv.size[0];
	const size_t J = cfg.Ediv.size[1];
	const size_t K = cfg.Ediv.size[2];

	const int gid = cfg.grpE.gid;
	const int nid = cfg.grpE.nid;

	SklSolverBase* solver = cfg.solver;

	const CfgElem* fldout = (output)? output->GetElemFirst("field"): NULL;
	if (!fldout) { return; }

	for (const CfgElem* fld = output->GetElemFirst("field");
			fld; fld = output->GetElemNext(fld, "field")) {

		const CfgParam* fattr  = fld->GetParamFirst("attr");
		const CfgParam* target = fld->GetParamFirst("target");
		if (!(fattr && target)) {
			if (nid == 0) {
				MSG_WARNING("not found attr or target param on \
							metabolism/output/field");
			}
			continue;
		}

		const char *ptgt,*pattr;
		const string tgt (target->GetData(&ptgt)? ptgt:  NULL);
		const string attr(fattr->GetData(&pattr)? pattr: NULL);

		string name, fullpn;
		_split_tgtstr(ptgt, name, fullpn);
		const Metabolism* mtb = UtilGetMetabolismId(name);
		if (!mtb) { continue; }

		unsigned int s = MtbSim[mtb->id]->isExistEntityProperty(fullpn)? 1: 0;
		unsigned int f;
		BEGIN_CLOCK(comm)

#ifdef SPHERE175
		para_mng->Allreduce(&s,&f,1,SKL_ARRAY_DTYPE_UINT,SKL_BAND,gid);		//2009.12.07
#else
		ParaMngr.Allreduce(&s,&f,1,SKL_ARRAY_DTYPE_UINT,SKL_BAND,gid);
#endif

		END_CLOCK(comm_clk,comm)
		if (!f) {
			stringstream msg;
			msg << "field out tartget '" << ptgt << "' is no exist";
			if (nid == 0) { MSG_WARNING(msg.str()); }
			continue;
		}

#ifdef SPHERE175
		SklArayBase*   ary  = solver->SklAllocateArray(para_mng, pattr,		//2009.12.07
								SKL_CLASS_SCALAR3D, SKL_ARRAY_DTYPE_REAL,
								cfg.Ediv.size, gid);
#else
		SklArayBase*   ary  = solver->SklAllocateArray(pattr,
								SKL_CLASS_SCALAR3D, SKL_ARRAY_DTYPE_REAL,
								cfg.Ediv.size, gid);
#endif

#ifdef REAL_IS_DOUBLE
		unsigned long long fsz[3];
#else
		unsigned int fsz[3];
#endif
		for (int i = 0; i < 3; i++) { fsz[i] = cfg.Ediv.size[i]; }
		SklVoxDataSet* dSet = solver->InitFile(pattr,
								fsz,cfg.origin,cfg.outpitch,ary);

		s = ((ary && dSet)? 1:0);
		BEGIN_CLOCK(comm2)

#ifdef SPHERE175
		para_mng->Allreduce(&s,&f,1,SKL_ARRAY_DTYPE_UINT,SKL_BAND,gid);	//2009.12.07
#else
		ParaMngr.Allreduce(&s,&f,1,SKL_ARRAY_DTYPE_UINT,SKL_BAND,gid);
#endif

		END_CLOCK(comm_clk,comm2)
		if (!f) {
			stringstream msg;
			msg << "initialization of output file for '"
				<< ptgt << "' was failed";
			if (nid == 0) { MSG_WARNING(msg.str()); }
			continue;
		}

		MetaboOutDataMap.insert(OutDataMapT::value_type(tgt, dSet));
	}
#endif // _WITHOUT_ECELL
}

void MetaboSolv::_init_probe(const CfgElem* output, const bool init_file)
{
#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

	if (!output) { return; }

#ifndef _WITHOUT_ECELL
	////////////////////////////////////////////////////////////////////////
	//
	// probe output setting
	//
	const size_t I = cfg.Ediv.size[0];
	const size_t J = cfg.Ediv.size[1];
	const size_t K = cfg.Ediv.size[2];

	const int nid = cfg.grpE.nid;
	const int gid = cfg.grpE.gid;

	for (const CfgElem* prb = output->GetElemFirst("probe");
			prb; prb = output->GetElemNext(prb, "probe")) {
		//
		MetaboProbeT probe;
		float coord[3];
		const CfgParam* px = prb->GetParamFirst("px");
		const CfgParam*	py = prb->GetParamFirst("py");
		const CfgParam* pz = prb->GetParamFirst("pz");
		const bool flg =   ((px)? px->GetData(&coord[0]): false)
						&& ((py)? py->GetData(&coord[1]): false)
						&& ((pz)? pz->GetData(&coord[2]): false);

		if (!flg) {
			stringstream msg;
			msg << "illegal cood for probe point";
			if (nid == 0) { MSG_WARNING(msg.str()); }
			continue;
		}

		probe.x = coord[0];
		probe.y = coord[1];
		probe.z = coord[2];

		probe._i = static_cast<unsigned int>(probe.x / cfg.pitch[0]);
		probe._j = static_cast<unsigned int>(probe.y / cfg.pitch[1]);
		probe._k = static_cast<unsigned int>(probe.z / cfg.pitch[2]);

		const int C = (cfg.Parallel)? 1: 0;
		probe.i = probe._i - C * cfg.Ediv.head[0];
		probe.j = probe._j - C * cfg.Ediv.head[1];
		probe.k = probe._k - C * cfg.Ediv.head[2];

		const bool probe_in_domain = (
									(probe._i < cfg.size[0])
								 && (probe._j < cfg.size[1])
								 && (probe._k < cfg.size[2]));
		const bool probe_in_local_domain = (
									(probe.i < cfg.Ediv.size[0])
								 && (probe.j < cfg.Ediv.size[1])
								 && (probe.k < cfg.Ediv.size[2]));

		// exchange node ID whitch has probed cell
		int z = (probe_in_local_domain)? nid: 0;
		BEGIN_CLOCK(comm)

#ifdef SPHERE175
		para_mng->Allreduce(&z,&(probe.nID),1,SKL_ARRAY_DTYPE_INT,SKL_SUM,gid);	//2009.12.07
#else
		ParaMngr.Allreduce(&z,&(probe.nID),1,SKL_ARRAY_DTYPE_INT,SKL_SUM,gid);
#endif

		END_CLOCK(comm_clk,comm)

		if ((nid != probe.nID) && (nid != 0)) { continue; }

		if (!probe_in_domain) { // out of domain!!
			stringstream msg;
			msg << "probe point ("
				<< probe.x << ','
				<< probe.y << ','
				<< probe.z << ')'
				<< " is out of domain";
			MSG_WARNING(msg.str());
			continue;
		}

		const char*     fn    = NULL;
		const CfgParam* fname = prb->GetParamFirst("fname");
		fn = ((fname) && (fname->GetData(&fn)))?  fn: NULL; 
		
		if (!fn) {
			stringstream msg;
			msg << "probe point ("
				<< probe.x << ','
				<< probe.y << ','
				<< probe.z << ')'
				<< " has no filename";
			if (nid == probe.nID) { MSG_WARNING(msg.str()); }
			continue;
		}
		probe.fname = fn;

		for (const CfgParam* obj = prb->GetParamFirst("target");
				obj; obj = prb->GetParamNext(obj, "target")) {

			const char* objname = NULL;
			if (!(obj->GetData(&objname) && objname)) { continue; }

			string matname, fullpn;
			_split_tgtstr(objname, matname, fullpn);

			// set logger
			if (nid == probe.nID) {
				const Metabolism* mtb = UtilGetMetabolismId(matname);
				if (!cfg.EmatAss[mtb->mat->id]) { continue; } // no assigned

				emcsim* sim = reinterpret_cast<emcsim*>(EmcSim->val(probe.i,probe.j,probe.k,mtb->id));
				if (!(sim && sim->isExistEntityProperty(fullpn))) {
					stringstream msg;
					msg << "probe target '" << objname << "' is not exist";
					MSG_WARNING(msg.str());
					continue;
				}
				sim->createLogger(fullpn);
			}

			probe.obj.push_back(MtbPrbTgtObjT(matname, fullpn));
		}

		if (!probe.obj.size()) {
			stringstream msg;
			msg << "probe point ("
				<< probe.x << ','
				<< probe.y << ','
				<< probe.z << ')'
				<< " has no target";
			if (nid == probe.nID) { MSG_WARNING(msg.str()); }
			continue;
		}

		MetaboProbeList.push_back(probe);

		// node ID is 0, open output file
		if ((nid == 0) && init_file) {
			ofstream of(probe.fname.c_str());

			of << "# time, x, y, z, i, j, k, ";
			for (vector<MtbPrbTgtObjT>::iterator i = probe.obj.begin();
					i != probe.obj.end(); i++) {

				of << '"' << i->first << ':' << i->second << "\", ";
			}
			of << endl;
			of.close();
		}
	}
#endif // _WITHOUT_ECELL
}

int MetaboSolv::MetaboSolvInitialize(const CfgElem* sim,
				S4D_REAL* res, const S4D_REAL* vfm, const S4D_REAL* vfim,
				const S4D_REAL* s,
				const S4D_REAL* nx, const S4D_REAL* ny, const S4D_REAL* nz,
				const S4D_REAL* p0,
				const S4D_REAL* smc,
				const S4D_REAL* smx, const S4D_REAL* smy, const S4D_REAL* smz)
{

	Vfm  = vfm;
	Vfim = vfim;
	P0   = p0;
	E    = res;
	Sm   = s;
	Nx   = nx;
	Ny   = ny;
	Nz   = nz;
	SmC  = smc;
	SmX  = smx;
	SmY  = smy;
	SmZ  = smz;
	EmcSim = NULL;
	SimMsk = NULL;

	const CfgElem*  mtb    = sim? sim->GetElemFirst ("metabolism"): NULL;
	const CfgParam* exe    = mtb? mtb->GetParamFirst("execution") : NULL;
	const CfgElem*  output = mtb? mtb->GetElemFirst ("output")    : NULL;

	const vxsize_t* sz = cfg.Ediv.size;
	const vxsize_t  I  = sz[0];
	const vxsize_t  J  = sz[1];
	const vxsize_t  K  = sz[2];

	// set flag, it means enable metabolism simulation
	const char* exc = NULL;
	enableMetabo = (exe && exe->GetData(&exc) && (strcmp(exc, "true") == 0));
	if (!enableMetabo) { return true; }

	FusArea.resize(cfg.MatCount);
	for (size_t m = 0; m < cfg.MatCount; m++) {
		FusArea[m].fmol = new SKL_REAL[cfg.MolCount];
	}

#ifndef _NO_MESSAGE
	MESSAGE("Allocate MetabolismSolver Local Fields...");
#else
	MESSAGE("Allocate MetabolismSolver Local Fields...",0);
#endif //_NO_MESSAGE
	_allocate_fields (I,J,K);

#ifndef _NO_MESSAGE
	MESSAGE("Initialize Fusion Setting...");
#else
	MESSAGE("Initialize Fusion Setting...",0);
#endif //_NO_MESSAGE
	_init_fusion_mask(I,J,K);

#ifndef _NO_MESSAGE
	MESSAGE("Construct E-Cell...");
#else
	MESSAGE("Construct E-Cell...",0);
#endif //_NO_MESSAGE
	_split           (I,J,K);
	_construct_ecell (I,J,K);

#ifndef _NO_MESSAGE
	MESSAGE("Binding for E-Cell Logger...");
#else
	MESSAGE("Binding for E-Cell Logger...",0);
#endif //_NO_MESSAGE
	_bind (I,J,K); // initial binding for logger

	_init_fldout(output, !cfg.CheckMode);
	_init_probe (output, !cfg.CheckMode);

	// output initial data
	_output_field(I,J,K,0,0);

	return 1;
}

int MetaboSolv::MetaboSolvLoop(const SKL_REAL time, const unsigned int step)
{

	if (!enableMetabo) { return 0; }

	BEGIN_CLOCK(loop)

	const size_t &I = cfg.Ediv.size[0];
	const size_t &J = cfg.Ediv.size[1];
	const size_t &K = cfg.Ediv.size[2];

//	_split   (I,J,K);
	_refusion(I,J,K);
	_bind    (I,J,K);
	_run     (I,J,K);
	_emitt   (I,J,K);
	_diff    (I,J,K);
	_post    (I,J,K);

	END_CLOCK(clock,loop)

	// metabo local field output
	DB2(MESSAGE("Output Metabo Filed Files"));
	_output_field(I,J,K,time,step);

	return 1;
}

int MetaboSolv::MetaboSolvPost()
{
	const unsigned int* sz = cfg.Ediv.size;
	const unsigned int  I  = sz[0];
	const unsigned int  J  = sz[1];
	const unsigned int  K  = sz[2];

	if (!enableMetabo) { return 1; }

	// emcsim local probe output
	_probe_output();
	
	for (size_t matid = 0; matid < cfg.MatCount; matid++) {
#ifndef _WITHOUT_ECELL
		delete   FusArea[matid].sim;
		delete[] FusArea[matid].fmol;
		FusArea[matid].sim  = NULL;
		FusArea[matid].fmol = NULL;
#endif // _WITHOUT_ECELL
	}

	for (size_t mtbid = 0; mtbid < cfg.MtbCount; mtbid++) {
		for (int k = 0; k < K; k++) {
		for (int j = 0; j < J; j++) {
		for (int i = 0; i < I; i++) {
#ifndef _WITHOUT_ECELL
			delete reinterpret_cast<emcsim*>(EmcSim->val(i,j,k,mtbid));
			EmcSim->val(i,j,k,mtbid) = NULL;
#endif // _WITHOUT_ECELL
		}}}

#ifndef _WITHOUT_ECELL
		delete MtbSim[mtbid];
		MtbSim[mtbid] = NULL;
#endif // _WITHOUT_ECELL
	}

	delete EmcSim;
	delete SimMsk;
	delete FusMsk;
	delete EVmdt;
	delete fm;

	EmcSim = NULL;
	SimMsk = NULL;
	FusMsk = NULL;
	EVmdt  = NULL;
	fm     = NULL;

	return 1;
}

int MetaboSolv::chkCfg() 
{
#ifdef SPHERE175
	para_cmp = cfg.solver->SklGetParaComponent();		//2009.12.07
	para_mng = para_cmp->GetParaManager();			//2009.12.07
#endif

#ifdef SPHERE175
	if (para_cmp->GetMyID(cfg.grpE.gid) != 0) { return 1; }	//2009.12.07
#else
	if (ParaMngr.GetMyID(cfg.grpE.gid) != 0) { return 1; }
#endif

#ifndef _NO_MESSAGE
	MESSAGE("**** Configuration of Metabolism Solver ****");
	MESSAGE(string("Metabolome Simulation is ")
				+ string_cast(enableMetabo, ios::boolalpha));
#else
	MESSAGE("**** Configuration of Metabolism Solver ****",0);
	MESSAGE(string("Metabolome Simulation is ")
				+ string_cast(enableMetabo, ios::boolalpha),0);
#endif //_NO_MESSAGE
	if (!enableMetabo) { return 0; }

	//////////////////////////////////////////////////////////////////////////
	// output Fieldout Setting
	stringstream fldout;
	fldout << "Field Output Setting" << endl;
	fldout << setw(80) << setfill('=') << "" << setfill(' ') << endl;
	fldout << setfill(' ')
		<< setw(3)  << left << ' '
		<< setw(30) << left << "material"  << "| "
		<< setw(30) << left << "File Base Name" << "| "
		<< endl;
	fldout << setw(80) << setfill('-') << "" << setfill(' ') << endl;
	for (OutDataMapT::iterator od = MetaboOutDataMap.begin();
			od != MetaboOutDataMap.end(); od++) {

		fldout
			<< setw(3)  << left << '*'
			<< setw(30) << od->first << "| "
			<< setw(30) << od->second->GetFileName() << "| "
			<< endl;
	}
	fldout << setw(80) << setfill('=') << "" << setfill(' ') << endl;

	//////////////////////////////////////////////////////////////////////////
	// output Probeout Setting
	stringstream prbout;
	prbout << "Probe Output Setting" << endl;
	prbout << setw(80) << setfill('=') << "" << setfill(' ') << endl;
	prbout << setw(40) << left << " cood";
	prbout << setw(20) << left << " filename" << endl;
	for (MetaboProbeListT::iterator prb = MetaboProbeList.begin();
			prb != MetaboProbeList.end(); prb++) {

		prbout << setw(80) << setfill('-') << "" << setfill(' ') << endl;

		prbout
			<< setw(13) << right << showpoint << prb->x
			<< setw(13) << right << showpoint << prb->y
			<< setw(13) << right << showpoint << prb->z
			<< setw(2)  << "";
		prbout
			<< setw(22) << left << prb->fname
			<< endl;

		prbout << " Targets : ";
		for (vector<MtbPrbTgtObjT>::iterator obj = prb->obj.begin();
				obj != prb->obj.end(); obj++) {
			prbout <<  obj->first  << ':' <<  obj->second << ", ";
		}
		prbout << endl;
	}
	prbout << setw(80) << setfill('=') << "" << setfill(' ') << endl;
	
	//////////////////////////////////////////////////////////////////////////
#ifndef _NO_MESSAGE
	MESSAGE(fldout.str());
	MESSAGE(prbout.str());
#else
	MESSAGE(fldout.str(), 0);
	MESSAGE(prbout.str(), 0);
#endif //_NO_MESSAGE

	return 1;
}


