//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _SKL_FORTRAN_FUNC_POTATION_H_
#define _SKL_FORTRAN_FUNC_POTATION_H_
#include "Skl.h"

extern "C" {
/* USER WRITE
   if using fortran subroutine,
   Please write fortran subroutine here.
   # The function described here belongs to the name scope of C.
   # Therefore, The identifier of Solver is recommended to be added to
   # the function name to distinguish from other functions.
*/

   void
   clear4s_(const int *imax, const int *jmax, const int *kmax, const int *vc,
          const SKL_REAL *p);

   void
   clear4d_(const size_t *imax, const size_t *jmax, const size_t *kmax, const size_t *nn,
            const int *vc, const SKL_REAL *p);

   void
   setcoef_(const int *imax, const int *jmax, const int *kmax, const int *vc,
            SKL_REAL *diffcoef, const SKL_REAL *membarea, const SKL_REAL *membarea2,
            const SKL_REAL *membnomalx,
            const SKL_REAL *membnomaly, const SKL_REAL *membnomalz,
            SKL_REAL *cx, SKL_REAL *cy, SKL_REAL *cz, SKL_REAL *coefdvdt);

   void
   setcoefreadmembarea_(const int *imax, const int *jmax, const int *kmax, const int *vc,
            SKL_REAL *diffcoef, const SKL_REAL *smc,
            const SKL_REAL *smx, const SKL_REAL *smy, const SKL_REAL *smz,
            const SKL_REAL *membnomalx,
            const SKL_REAL *membnomaly, const SKL_REAL *membnomalz,
            SKL_REAL *cx, SKL_REAL *cy, SKL_REAL *cz, SKL_REAL *coefdvdt);

   void
   inithh_(const int *imax, const int *jmax, const int *kmax, const int *vc,
           SKL_REAL *M, SKL_REAL *N, SKL_REAL *H, SKL_REAL *V,
           SKL_REAL *mm, SKL_REAL *nn, SKL_REAL *hh,
           SKL_REAL *coefdvdt, const SKL_REAL *val);

   void
   setopenarea_(SKL_REAL *ax, SKL_REAL *ay, SKL_REAL *az,
                 const SKL_REAL *AX, const SKL_REAL *AY, const SKL_REAL *AZ,
                 SKL_REAL *coefdvdt, 
                 const int *imax, const int *jmax, const int *kmax,
                 const int *vc);
   void
   calccoef_hh_(SKL_REAL* cx, SKL_REAL* cy, SKL_REAL* cz,
              const SKL_REAL* _Cx, const SKL_REAL* _Cy, const SKL_REAL* _Cz,
              const int* nID,  const int* m, const int* mols,
              const int* imax, const int* jmax, const int* kmax, const int* vc);

   void
   setbndcoef_hh_(SKL_REAL* _Cx, SKL_REAL* _Cy, SKL_REAL* _Cz,
               const int* nID,  const int* m, const int* mols,
               const int* imax, const int* jmax, const int* kmax, const int* vc);

   void
   differential_hh_(int* Flag, const SKL_REAL* P,
                 const SKL_REAL* cx, const SKL_REAL* cy, const SKL_REAL* cz,
                 const SKL_REAL* ax, const SKL_REAL* ay, const SKL_REAL* az,
                 SKL_REAL* diffx, SKL_REAL* diffy, SKL_REAL* diffz,
                 const int* imax, const int* jmax, const int* kmax, const int* vc,
                 const SKL_REAL* dxi, const SKL_REAL* dyi, const SKL_REAL* dzi );


   void
   calchh_(const int *imax, const int *jmax, const int *kmax, const int *vc,
           SKL_REAL *Cm, SKL_REAL *gK, SKL_REAL *gNa, SKL_REAL *gL,
           SKL_REAL *vK, SKL_REAL *vNa, SKL_REAL *vL, const SKL_REAL *dt,
           const SKL_REAL *val, SKL_REAL *mm, SKL_REAL *nn, SKL_REAL *hh, 
           SKL_REAL *dvdt, SKL_REAL *coefdvdt);

   void
   dvdtdvcm_(const int *imax, const int *jmax, const int *kmax, const int *vc,
           SKL_REAL *Cm, 
           SKL_REAL *dvdt, SKL_REAL *coefdvdt);

   void
   exptimhh_(const SKL_REAL *dt, const int *imax, const int *jmax, const int *kmax,
             const int *vc, 
             SKL_REAL *p1, const SKL_REAL *dvdt);

   void
   exptim_diff_(const SKL_REAL *dt, const int *imax, const int *jmax, const int *kmax,
             const int *vc, 
             const SKL_REAL *dxi, const SKL_REAL *dyi, const SKL_REAL *dzi,
             const SKL_REAL *diffx, const SKL_REAL *diffy, const SKL_REAL *diffz, 
             const SKL_REAL *phii, SKL_REAL *p1, const SKL_REAL *val);

}

#endif //_SKL_FORTRAN_FUNC_POTATION_H_
