#include <Foundation/util.h>
#include <Foundation/error.h>

#include "PotentialSolv.h"
#include "MembranePotential.h"

bool MembranePotential::ParseXML(const CfgElem* memp, const CfgInfo& cfg)
{

       // Cm
       const CfgParam* poteParam = memp->GetParamFirst("Cm");
       if( poteParam ) {
           poteParam->GetData(&Cm);
       } else {
           Cm = 1.0;
       } 

       //gK
       poteParam = memp->GetParamFirst("gK");
       if( poteParam ) {
           poteParam->GetData(&gK);
       } else {
           gK = 36.0;
       } 

       //gNa
       poteParam = memp->GetParamFirst("gNa");
       if( poteParam ) {
           poteParam->GetData(&gNa);
       } else {
           gNa = 120.0;
       }

       //gL
       poteParam = memp->GetParamFirst("gL");
       if( poteParam ) {
           poteParam->GetData(&gL);
       } else {
           gL = 0.3;
       }

       //vK
       poteParam = memp->GetParamFirst("vK");
       if( poteParam ) {
           poteParam->GetData(&vK);
       } else {
           vK = -12.0;
       }

       //vNa
       poteParam = memp->GetParamFirst("vNa");
       if( poteParam ) {
           poteParam->GetData(&vNa);
       } else {
           vNa = 115.0;
       }

       //vL
       poteParam = memp->GetParamFirst("vL");
       if( poteParam ) {
           poteParam->GetData(&vL);
       } else {
           vL = 10.613;
       }

       //DiffCoef
       poteParam = memp->GetParamFirst("DiffCoef");
       if( poteParam ) {
           poteParam->GetData(&diffcoef);
       } else {
           diffcoef = 0.0;
       }

       //v
       poteParam = memp->GetParamFirst("V");
       if( poteParam ) {
           poteParam->GetData(&V);
       } else {
           V = 0.0;
       }

       //m
       poteParam = memp->GetParamFirst("M");
       if( poteParam ) {
           poteParam->GetData(&M);
       } else {
           M = 0.0;
       }
       //n
       poteParam = memp->GetParamFirst("N");
       if( poteParam ) {
           poteParam->GetData(&N);
       } else {
           N = 0.0;
       }

       //h
       poteParam = memp->GetParamFirst("H");
       if( poteParam ) {
           poteParam->GetData(&H);
       } else {
           H = 0.0;
       }

       //Iapp
       const CfgElem* IappElm = memp->GetElemFirst("Iapp");
       if( IappElm ) {
         IappTable* iapp = new IappTable;
         iapp->ParseXML(IappElm,cfg);
         IappList.push_back(iapp);
       }
       return true;

}
//
//
bool IappTable::ParseXML(const CfgElem* iappElm, const CfgInfo& cfg)
{

       // XML I,J,K
       //const CfgElem *indexElm = iappElm->GetElemFirst("Index");
       const CfgElem *indexElm = iappElm->GetElemFirst();
       while( indexElm ) {

          //DBX(MESSAGE("Iapp Index define"));
          const char* name = indexElm->GetName();
          if( strcmp(name,"Index" ) == 0 ) {

            int idx[3];
            if( SklUtil::getVecParams(indexElm,"I","J","K",idx)) 
               AddUniqueIndex(idx[0],idx[1],idx[2]);

          } else if( strcmp(name,"Coord") == 0 ) {

            SKL_REAL coord[3];
            if( SklUtil::getVecParams(indexElm,"X","Y","Z",coord) ) {
            // X,Y,Z -> I,J,K
               int idx[3];
               idx[0] = static_cast<unsigned int>(coord[0] / cfg.pitch[0]);
               idx[1] = static_cast<unsigned int>(coord[1] / cfg.pitch[1]);
               idx[2] = static_cast<unsigned int>(coord[2] / cfg.pitch[2]);

               AddUniqueIndex(idx,coord);

            }
          } else if( strcmp(name,"SPH") == 0 ) {
            printf("Elem name SPH\n");
            //const CfgParam* attrParam = sphElem->GetParamFirst("attr");
            const CfgParam* attrParam = indexElm->GetParamFirst("attr");
            while(attrParam)
            {
               const char* attr = NULL;
               if( attrParam->GetData(&attr) )
               sphAttr.push_back(attr);

               attrParam = indexElm->GetParamNext(attrParam,"attr");

            }
          }
          indexElm = iappElm->GetElemNext(indexElm);
       }

       // XML X,Y,Z
       /*
       const CfgElem *coordElm = iappElm->GetElemFirst("Coord");
       while( coordElm ) {

          DBX(MESSAGE("Iapp Coord define"));

          SKL_REAL coord[3];
          if( SklUtil::getVecParams(coordElm,"X","Y","Z",coord) ) { 
          // X,Y,Z -> I,J,K
            int idx[3];
            idx[0] = static_cast<unsigned int>(coord[0] / cfg.pitch[0]);
            idx[1] = static_cast<unsigned int>(coord[1] / cfg.pitch[1]);
            idx[2] = static_cast<unsigned int>(coord[2] / cfg.pitch[2]);

            AddUniqueIndex(idx,coord);

            coordElm = iappElm->GetElemNext(coordElm,"Coord");
          }
       }
       */
       // XML SPH
       /*
       const CfgElem* sphElem = iappElm->GetElemFirst("SPH");
       if( sphElem ) {
         // SPH ATTR 
         while( sphElem )
         {
            const CfgParam* attrParam = sphElem->GetParamFirst("attr");
            while(attrParam)
            {
               const char* attr = NULL;
               if( attrParam->GetData(&attr) )
               sphAttr.push_back(attr);

               attrParam = sphElem->GetParamNext(attrParam,"attr");
            }

            // search next elem
            sphElem = iappElm->GetElemNext(sphElem,"sph");
         }
       } 
       */
       // XML Iapp FixedValue
       Iapp=0.0;
       const CfgParam* iappPrm1 = iappElm->GetParamFirst("FixedValue");
       if( iappPrm1 ) iappPrm1->GetData(&Iapp);

       // XML Iapp TableValue
       const CfgParam* iappPrm2 = iappElm->GetParamFirst("TableValue");
       if( iappPrm1 && iappPrm2 ) {
          MSG_WARNING(UtilStrFormat("FixedValue and TableValue found FixedValue effective"));
          MSG_WARNING(UtilStrFormat("                          Iapp value : %e",Iapp));
          return true;
       }
       if( iappPrm2 ) {
          iappPrm2->GetData(&csv_fname);

          ifstream ifs;

          DBX(MESSAGE(UtilStrFormat("CSV FILE NAME : %s",csv_fname)));

          const char sep = ',';
          bool ret = true;
          try
          {
             ifs.open(csv_fname,ios::in);
             if( ifs.is_open() )
             {
                 string line;
                 while (getline(ifs, line))
                 {
                     string::size_type p1 = line.find_first_of(sep);
                     if( p1 == string::npos ) continue;
                     string::size_type p2 = line.find_first_of(sep, p1 + 1);
                     const string f = line.substr(0,p1);
                     const string s = line.substr(p1 + 1, p2 - p1 -1);
                     const SKL_REAL v1 = static_cast<SKL_REAL>(atof(f.c_str()));
                     const SKL_REAL v2 = static_cast<SKL_REAL>(atof(s.c_str()));

                     DBX(MESSAGE(UtilStrFormat("CSV FILE V1,V2 : %e %e",v1,v2)));

                     AddUnique(v1,v2);
                  }
              }
              else
              {
                  ret = false;
              }
          }
          catch(...)
          {
             ret = false;
          }
          ifs.close();
       }

       return true;
}
//
//
//
bool IappTable::AddUnique(const SKL_REAL &V1, const SKL_REAL &V2)
{
       bool exist = false;

       for( int i=0;i<IappTbl.size();i++ )
       {
          if( IappTbl.at(i).first == V1 && IappTbl.at(i).second == V2 )
          {
             exist = true;
             break;
          }
       }
       if( !exist )
       {
          IappTbl.push_back(RealPair(V1,V2));
          return true;
       }
       return false;
}
//
bool IappTable::AddUniqueIndex(const int &I, const int &J, const int &K)
{
       bool exist = false;
       for( int i=0;i<indexList.size();i++ ) 
       {
          if( I == indexList[i]->i &&
              J == indexList[i]->j &&
              K == indexList[i]->k ) {
              exist = true;
              break;
          }
        }
        if( !exist )
        {
           IappIndex *List = new IappIndex(I,J,K);
           indexList.push_back(List);
           return true;
        }
        return false;
}
//
bool IappTable::AddUniqueIndex(int *idx, SKL_REAL *coord)
{

       bool exist = false;
       for( int i=0;i<indexList.size();i++ )
       {
          if( idx[0] == indexList[i]->i &&
              idx[1] == indexList[i]->j &&
              idx[2] == indexList[i]->k ) {
              exist = true;
              break;
          }
        }
        if( !exist )
        {
           IappIndex *List = new IappIndex(idx,coord);
           indexList.push_back(List);
           return true;
        }
        return false;
}
//
SKL_REAL IappTable::GetTimeValueByLinear(SKL_REAL &xx)
{

       //SKL_REAL x = xx - deley;
       SKL_REAL x = xx;

       int ncnt = IappTbl.size();

       if( ncnt == 0 ) return Iapp;
       //if( x <= IappTbl.at(0).first ) return Iapp;
       if( x <= IappTbl.at(0).first ) return IappTbl.at(0).second;
       //if( x >= IappTbl.at(ncnt-1).first ) return Iapp;
       if( x >= IappTbl.at(ncnt-1).first ) return IappTbl.at(ncnt-1).second;

       for( int i=1;i<ncnt;i++ )
       {
          SKL_REAL x1 = IappTbl.at(i-1).first;
          SKL_REAL x2 = IappTbl.at(i  ).first;
          if( x < x2 )
          {
              SKL_REAL y1 = IappTbl.at(i-1).second;
              SKL_REAL y2 = IappTbl.at(i  ).second;
              if( x1==x2 ) return y2;
              if( y1==y2 ) return y2;
              SKL_REAL a = (y1-y2)/(x1-x2);
              SKL_REAL b = y1 - a * x1;
              return a * x + b;
          }
       }
       return IappTbl.at(ncnt-1).second;
}
//
bool MembranePotential::GetPotentialMat(const char* name)
{

        mat1 = NULL;
        mat2 = NULL;

        const char colon =':';
        string nm = string(name);

        string::size_type p1 = nm.find_first_of(colon);
        if( p1 == string::npos ) return false;
        string::size_type p2 = nm.find_first_of(colon);
        const string n1 = nm.substr(0,p1);
        const string n2 = nm.substr(p1+1,p2-p1-1);

        mat1 = UtilGetMaterialId(n1);
        mat2 = UtilGetMaterialId(n2);

        if( mat1 == NULL || mat2 == NULL ) return false;

        DBX(MESSAGE(UtilStrFormat("membranePotetial_setting mat1 : %s mat2 : %s",mat1->name.c_str(),
                                                                                 mat2->name.c_str())));

        return true;
}
