//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __MEMBRANE_POTENTIAL_H__
#define __MEMBRANE_POTENTIAL_H__

#include <Foundation/datatype.h>
#include <Foundation/const.h>


//struct
struct IappIndex
{
           int i,j,k;
           int oi,oj,ok;
           SKL_REAL X,Y,Z;
           bool setcoord;
           const char* attr;

           IappIndex()
           {
               i=j=k    = 0;
               oi=oj=ok = 0;
               X=Y=Z    = SKL_REAL(0.0);
               setcoord = false;
               attr     = NULL;
           } 

           IappIndex(int _I, int _J, int _K)
           {
               i = oi = _I;
               j = oj = _J;
               k = ok = _K;
               X=Y=Z    = SKL_REAL(0.0);
               setcoord = false;
               attr     = NULL;
           } 

           IappIndex(int* idx)
           {
               i = oi = idx[0];
               j = oj = idx[1];
               k = ok = idx[2];
               X=Y=Z    = SKL_REAL(0.0);
               setcoord = false;
               attr     = NULL;
           } 

           IappIndex (int* idx, SKL_REAL* coord)
           {
               i = oi = idx[0];
               j = oj = idx[1];
               k = ok = idx[2];
               X = coord[0];
               Y = coord[1];
               Z = coord[2];
               setcoord = true;
               attr     = NULL;
           }

           IappIndex(int _I, int _J, int _K, SKL_REAL* coord)
           {
               i = oi = _I;
               j = oj = _J;
               k = ok = _K;
               X = coord[0];
               Y = coord[1];
               Z = coord[2];
               setcoord = true;
               attr     = NULL;
           }

           IappIndex(int* idx, const char* sphattr) 
           {
               i = oi = idx[0];
               j = oj = idx[1];
               k = ok = idx[2];
               setcoord = false;
               attr     = sphattr;
           } 
};

struct IappTable 
{
           SKL_REAL Iapp;
           RealPairVector IappTbl;
           const char* csv_fname;

           vector<const char*> sphAttr;

           vector<IappIndex *> indexList;

           SKL_REAL GetTimeValueByLinear(SKL_REAL &time);
           bool AddUnique(const SKL_REAL &V1, const SKL_REAL &V2); 
           bool AddUniqueIndex(const int &I, const int &J, const int &K); 
           bool AddUniqueIndex(int *idx, SKL_REAL *coord); 
           bool ParseXML(const CfgElem* iappElm, const CfgInfo& cfg);

};

struct MembranePotential
{
           const char* name;
           int id;
           const Material* mat1;
           const Material* mat2;
           SKL_REAL Cm,gK,gNa,gL,vK,vNa,vL,diffcoef;
           SKL_REAL V,M,N,H;
           vector<IappTable*> IappList;

           bool ParseXML(const CfgElem* memP, const CfgInfo& cfg);
           bool GetPotentialMat(const char* name);
};

#endif /*__MEMBRANE_POTENTIAL_H__*/

