//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __POTATION_SOLV_H__
#define __POTATION_SOLV_H__

#include <Skl.h>
#include <SklUtil.h>

#include <iostream>
#include <cmath>
#include <cstring>
#include <map>
#include <cfloat>

#include <Foundation/datatype.h>
#include <Foundation/api.h>
#include "MembranePotential.h"

class PotentialSolv
{
protected:

        vector<MembranePotential *> h_MemPotTbl;                         

// member variables
	const CfgInfo& cfg;
        const int vc;

        int h_nID[6];

	// H-H parameters
	bool enableHH;
        int diffTypeFlag;

	// for time mesurement
	double clock;
	double comm_clk;
	double wait_clk;

        //dataclass
        // data class
        S4D_REAL *h_p1;  //
        const S4D_REAL *h_p0;  //
        const S4D_REAL *h_phi; //
        const S4D_REAL *h_phii;//
        const S4D_REAL *h_membarea; //
        const S4D_REAL *h_membnomalx; //
        const S4D_REAL *h_membnomaly; //
        const S4D_REAL *h_membnomalz; //
        S4D_REAL *h_diffx;     // destination
        S4D_REAL *h_diffy;     // destination
        S4D_REAL *h_diffz;     // destination

        const S4D_INT *h_BCx;
        const S4D_INT *h_BCy;
        const S4D_INT *h_BCz;

	const S4D_REAL *h_smc;	// Membrane Area Center
	const S4D_REAL *h_smx;	// Membrane Area X face
	const S4D_REAL *h_smy;	// Membrane Area Y face
	const S4D_REAL *h_smz;	// Membrane Area Z face

        S4D_REAL *h_val;
        S4D_REAL *h_CX, *h_CY,*h_CZ;
        S4D_REAL *h_cx, *h_cy,*h_cz;
        const S4D_REAL *h_AX, *h_AY,*h_AZ;
        S4D_REAL *h_ax, *h_ay,*h_az;
        S4D_REAL *h_mm, *h_nn,*h_hh;
        S4D_REAL *h_coefdvdt;
        S4D_REAL *h_dvdt;

        SKL_REAL h_M,h_N,h_H,h_V;
        SKL_REAL h_diffcoef;

        SKL_REAL h_Cm;
        SKL_REAL h_gK,h_gNa,h_gL;
        SKL_REAL h_vK,h_vNa,h_vL;

        // DEBUG FLAG
        bool hh_calc;

	// for parallel
	SklParaComponent* para_cmp;
	SklParaManager*   para_mng;

        //molecular V0
        const Molecular* h_mol;

// member functions
	bool Allocate();
	bool printLog(int nid=-1);

        int SetBndCell();
        SKL_REAL CalcBndCell(
                const int bc, const SKL_REAL c,
                const SKL_REAL dx, const int face);


public:

// member functions
	PotentialSolv(const CfgInfo& _cfg);
	virtual ~PotentialSolv();
	int PotentialInitialize(const CfgElem* simElem, const CfgElem* HHRoot,
                               S4D_REAL *P1, const S4D_REAL *P0, 
                               const S4D_REAL *AX, const S4D_REAL *AY, const S4D_REAL *AZ,
                               const S4D_REAL *PHI, const S4D_REAL *PHII,
                               const S4D_REAL *MEMBAREA, const S4D_REAL *MEMBNOMALX,
                               const S4D_REAL *MEMBNOMALY, const S4D_REAL *MEMBNOMAL_Z,
                               S4D_REAL *DIFFX, S4D_REAL *DIFFY, S4D_REAL *DIFFZ,
                               S4D_REAL *COEFX, S4D_REAL *COEFY, S4D_REAL *COEFZr,
                               const S4D_INT *BCX, const S4D_INT *BCY, const S4D_INT *BCZ,
                               const S4D_REAL *smc,
                               const S4D_REAL *smx, const S4D_REAL *smy, const S4D_REAL *smz);
	int PotentialLoop(const SKL_REAL dt,  const unsigned int step);
	int PotentialPost();
	bool ParseXML(const CfgElem* HHElem);
        //bool GetPotentialMat(const char* name, const Material* mat1, const Material* mat2);
	bool chkCfg();

        bool chkPoten(const int matid);
        bool LoadPositionSPH();

	double GetClock()   const { return clock;    };
	double GetCommClk() const { return comm_clk; };
	double GetWaitClk() const { return wait_clk; };

        int ExptimHH(const SKL_REAL dt);

        void splitOutFileName( const char *name, string &typ, string &mat_name_1,
                               string &mat_name_2);
        TypeEnum checkPotenType( const char *name );
        const Element* getPotenElem( const char *name);
        S3D_REAL* getPotenArray( const char *name, S3D_REAL *s3d, S4D_REAL* P0, int eqnId);

        int dbunit;
        char* dbname;

};

#endif /*__POTATION_SOLV_H__*/

