//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#ifndef __WARP_BASE_H__
#define __WARP_BASE_H__

#include "WarpList.h"

class WarpBase
{
protected:

// member variables
	const CfgInfo& cfg;

	// config XML elem
	int         m_elemID;
	const char* m_elemName;

	// xml values
	WarpParamList  m_paramList;   // param list
	WarpTableList  m_tableList;   // table list

	// for Parallel
	SklParaComponent* para_cmp;   // parallel component
	SklParaManager*   para_mng;   // parallel manager

	// counter
	int m_counter;

// member function


public:

// member functions
	WarpBase(const CfgInfo& _cfg, int cnt=(-1));
	virtual ~WarpBase();
	bool ParseXML(const CfgElem* elemRoot);
	virtual bool printLog(stringstream& wout, int ts, int nid=-1);

	int GetElemID(){ return m_elemID; };
	const char* GetElemName(){ return m_elemName; };

	// for table
	SKL_REAL GetTableValue(const char* tableName, SKL_REAL x);

	// for param
	int      GetIntParam(const char* name, int def=0);
	SKL_REAL GetRealParam(const char* name, SKL_REAL def=SKL_REAL(0.0));
	string   GetStringParam(const char* name, string def=string(""));
	string   GetStringParam(const char* name, WarpList<string>& choice, string def=string(""));

	// for check param
	E_WARP_VALUE_TYPE CheckValueType();
	E_WARP_OP         CheckOpeType();
	E_WARP_CMP        CheckCompType();

};

#endif /*__WARP_BASE_H__*/

