/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */
#include "WarpGroup.h"
#include "FortranFuncWarp.h"

/////////////////////////////////////////////////////////////////////////////
// Constructor
//
WarpGroup::WarpGroup(const CfgInfo& _cfg, int cnt)
: WarpBase(_cfg, cnt)//, m_tempArea(_cfg)
{
	// m_emcsim = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// Destractor
//
WarpGroup::~WarpGroup()
{
	int i;
	for( i=0;i<m_warpInfo.size();i++ )
	{
		delete m_warpInfo[i];
	}
	m_warpInfo.clear();
}

/////////////////////////////////////////////////////////////////////////////
// Parse XML config
//
bool WarpGroup::ParseXML(const CfgElem* elemRoot)
{
	if( !elemRoot ) return false;

	// parse param by BaseClass function
	if( !WarpBase::ParseXML(elemRoot) )
	{
		return false;
	}

  DB1(MESSAGE(UtilStrFormat("WarpGroup::ParseXML - %s Elem(id=%d)",m_elemName, m_elemID)));

	// search TempArea Elem
	const CfgElem* tempAreaElem = elemRoot->GetElemFirst("TempArea");

	// temporary area initialize
	if( !m_tempArea.Initialize(tempAreaElem, GetWarpType() )){
		MSG_ERROR(UtilStrFormat("Temporary area Initialize error"));
		return false;
	}

	// search WarpInfo Elem
	int cnt = 0;
	const CfgElem* elem = elemRoot->GetElemFirst("WarpInfo");
	while( elem )
	{
		// instance
		WarpInfo* warpInfo = new WarpInfo(cfg, ++cnt);
		if( !warpInfo->ParseXML(elem,&m_tempArea) )
		{
  		MSG_ERROR(UtilStrFormat("configuration XML parse error in %s Elem(id=%d)",m_elemName, m_elemID));
			delete warpInfo;
			return false;
		}

		// check warp info Source & Distination
		if( !warpInfo->CheckWarpInfo( GetWarpType() ) ){
			// print ErrorMessage in CheckWarpInfo function
			return false;
		}

		// add to vector
		m_warpInfo.push_back(warpInfo);

		// next elem
		elem = elemRoot->GetElemNext(elem,"WarpInfo");
	}

	string msg;
	msg += UtilStrFormat("WarpGroup::ParseXML - EqnCount=%d \n",cfg.EqnCount);
	for( int e=0;e<cfg.EqnCount;e++ )
	{
		bool bb = true;
		if( !cfg.PeqnAss[e] ) bb = false;
		msg += UtilStrFormat("  %s, %s\n", cfg.EqnTbl[e]->name.c_str(), string_cast(bb, ios::boolalpha).c_str());
	}
//  DB1(MESSAGE(UtilStrFormat("WarpGroup::ParseXML - EqnCount=%d ",cfg.EqnCount)));
  DB1(MESSAGE(msg));

	return true;
}

E_WARP_TYPE WarpGroup::GetWarpType()
{
	string type = GetStringParam("type");
	if( !strcasecmp(type.c_str(),"Direct"   ) ) return E_WARP_DIRECT;
	if( !strcasecmp(type.c_str(),"Mediation") ) return E_WARP_MEDIATION;
	if( !strcasecmp(type.c_str(),"Capillary") ) return E_WARP_CAPILLARY;
	return E_WARP_USER;
}

/////////////////////////////////////////////////////////////////////////////
// Check E-CELL execution define
//
// bool WarpGroup::checkEcellExec()
// {
	// string eml = m_paramList.GetStringByName("metabolism");
	// const char* eml = NULL;
	// const CfgElem* tempAreaElem = elemRoot->GetElemFirst("TempArea");
	// const CfgParam* MP = tempAreaElem->GetParamFirst("metabolism");
	// string emlfile = (MP && MP->GetData(&eml) && (eml != NULL))? string(eml): string("");

	// if( emlfile.length() == 0 ) return false;
	// if( GetWarpType() != E_WARP_MEDIATION ) return false;
	// return true;
// }

bool WarpGroup::printLog(stringstream& wout, int ts, int nid)
{
	int i;

	// base
	WarpBase::printLog(wout,ts,nid);

	ts += WARP_TS;

	m_tempArea.printLog(wout, ts, nid);

	for( i=0;i<m_warpInfo.size();i++ )
	{
		if( !m_warpInfo[i]->printLog(wout,ts,nid) )
		{
			return false;
		}
	}

	return true;
}


