/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */
#include "WarpGroup.h"
#include "FortranFuncWarp.h"

bool WarpGroup::Calc()
{
	DB1(MESSAGE(UtilStrFormat("WarpGroup::Calc (id=%d)",m_elemID)));

	// type
	E_WARP_TYPE warpType = GetWarpType();

	bool ret = false;
	if( warpType == E_WARP_DIRECT )
	{
		ret = CalcDirect();
	}
	else if( warpType == E_WARP_MEDIATION )
	{
		ret = CalcMediation();
	}
	else if( warpType == E_WARP_CAPILLARY )
	{
		ret = CalcCapillary();
	}
	else
	{
		ret = CalcUser();
	}

stringstream wout;
wout << string("warp temp area") << endl;
for( int i=0;i<m_tempArea.Count();i++ )
{
	// m_tempArea parameter is ...
	// const Molecular* molecular, SKL_REAL mol

	// wout << "  " << setw(20) << left << m_tempArea[i].name;
	wout << "  " << setw(20) << left << m_tempArea[i].molecular->name;
	wout << ": " << UtilStrFormat("%.6e",m_tempArea[i].mol);
	// wout << ": " << UtilStrFormat("%.6e",m_tempArea[i].volume);
	wout << endl;
}
DB1(MESSAGE(wout.str()));

	return ret;
}

/////////////////////////////////////////////////////////////////////////////
// for Direct
//
bool WarpGroup::CalcDirect()
{
	DB1(MESSAGE(UtilStrFormat("WarpGroup::CalcDirect")));
	for( int i=0;i<m_warpInfo.size();i++ )
	{
		WarpInfo *wInfo = m_warpInfo[i];
		if( !wInfo ) continue;

		//WarpInfo_CalcDirect.C
		if( !wInfo->CalcDirect(&m_tempArea) )
		{
			return false;
		}
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
// for Mediation
//
bool WarpGroup::CalcMediation()
{
	DB1(MESSAGE(UtilStrFormat("WarpGroup::CalcMediation")));
	// Temporary Area : Run E-Cell
	m_tempArea.RunECell();

	for( int i=0;i<m_warpInfo.size();i++ )
	{
		WarpInfo *wInfo = m_warpInfo[i];
		if( !wInfo ) continue;

		//WarpInfo_CalcMediation.C
		if( !wInfo->CalcMediation(&m_tempArea) )
		{
			return false;
		}
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
// for Capillary
//
bool WarpGroup::CalcCapillary()
{
	DB1(MESSAGE(UtilStrFormat("WarpGroup::CalcCapillary")));
	for( int i=0;i<m_warpInfo.size();i++ )
	{
		WarpInfo *wInfo = m_warpInfo[i];
		if( !wInfo ) continue;
		// DB1(MESSAGE(UtilStrFormat("WarpGroup::CalcCapillary2")));

		//WarpInfo_CalcCapillary.C
		if( !wInfo->CalcCapillary(&m_tempArea) )
		{
			return false;
		}
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
// for User Define
//
bool WarpGroup::CalcUser()
{
	DB1(MESSAGE(UtilStrFormat("WarpGroup::CalcUser")));
	for( int i=0;i<m_warpInfo.size();i++ )
	{
		WarpInfo *wInfo = m_warpInfo[i];
		if( !wInfo ) continue;

		//WarpInfo_CalcUser.C
		if( !wInfo->CalcUser(&m_tempArea) )
		{
			return false;
		}
	}

	return false;
}


