/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */
#include "WarpInfo.h"
#include "FortranFuncWarp.h"

/////////////////////////////////////////////////////////////////////////////
// Constructor
//
WarpInfo::WarpInfo(const CfgInfo& _cfg, int cnt)
: WarpBase(_cfg, cnt)
{
	m_srcMolInfo = NULL;
	m_dstMolInfo = NULL;
	m_srcTrigger = NULL;
	m_dstTrigger = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// Destractor
//
WarpInfo::~WarpInfo()
{
	if( m_srcMolInfo ) delete m_srcMolInfo;
	if( m_dstMolInfo ) delete m_dstMolInfo;
	if( m_srcTrigger ) delete m_srcTrigger;
	if( m_dstTrigger ) delete m_dstTrigger;
}

/////////////////////////////////////////////////////////////////////////////
// Parse XML config
//
bool WarpInfo::ParseXML(const CfgElem* elemRoot, WarpTemporaryArea* tempArea)
{
	if( !elemRoot ) return false;

	// parse param by BaseClass function
	if( !WarpBase::ParseXML(elemRoot) )
	{
		return false;
	}

  DB1(MESSAGE(UtilStrFormat("WarpInfo::ParseXML - %s Elem(id=%d)",m_elemName, m_elemID)));

	const char* molInfoName[4] ={ "SourceInfo", "DistinationInfo", "SourceTrigger", "DistinationTrigger" };

	WarpMolInfo** molInfoList[4] = {&m_srcMolInfo,&m_dstMolInfo,&m_srcTrigger,&m_dstTrigger};

	// search MolInfo Elem
	for( int i=0;i<4;i++ )
	{
		// search element
		const CfgElem* elem = elemRoot->GetElemFirst(molInfoName[i]);
		// if( i<2 && !elem )
		// {
 		//	MSG_ERROR(UtilStrFormat("%s Elem is undefined in %s Elem(id=%d)", molInfoName[i], m_elemName, m_elemID));
		//	return false;
		// }
		if( !elem ) continue;

		WarpMolInfo* molInfo = new WarpMolInfo(cfg);
		if( !molInfo->ParseXML(elem) )
		{
			delete molInfo;
			return false;
		}
		*(molInfoList[i]) = molInfo;
	}

	// set temp area volume
/*
	const Equation* srcEqn = m_srcMolInfo->GetEquation();
	const Equation* dstEqn = m_dstMolInfo->GetEquation();
	string tempVolume = GetStringParam("TempVolume");
	if( !strcasecmp(tempVolume.c_str(),"Distination") )
	{
		SKL_REAL vol = m_dstMolInfo->GetAreaVolume();
		tempArea->SetVolume(srcEqn->name.c_str(),vol);
		tempArea->SetVolume(dstEqn->name.c_str(),vol);
	}
	else if( !strcasecmp(tempVolume.c_str(),"Fixed") )
	{
		SKL_REAL vol = GetRealParam("FixedTempVolume");
		tempArea->SetVolume(srcEqn->name.c_str(),vol);
		tempArea->SetVolume(dstEqn->name.c_str(),vol);
	}
	else
	{
		SKL_REAL vol = m_srcMolInfo->GetAreaVolume();
		tempArea->SetVolume(srcEqn->name.c_str(),vol);
		tempArea->SetVolume(dstEqn->name.c_str(),vol);
	}
*/

	return true;
}

bool WarpInfo::CheckWarpInfo(E_WARP_TYPE warpType)
{
	bool checkInfo = false;

	if(warpType == E_WARP_MEDIATION){
		if((!m_srcMolInfo) && (!m_dstMolInfo)){
			checkInfo = false;
			MSG_ERROR(UtilStrFormat("SourceInfo Elem and DistinationInfo Elem are undefined in %s Elem(id=%d)", m_elemName, m_elemID));
		}
		else{
			checkInfo = true;
		}
	}
	else{
		if((m_srcMolInfo) && (m_dstMolInfo)){
			checkInfo = true;
		}
		else if((!m_srcMolInfo) && (!m_dstMolInfo)){
			checkInfo = false;
			MSG_ERROR(UtilStrFormat("SourceInfo Elem and DistinationInfo Elem are undefined in %s Elem(id=%d)", m_elemName, m_elemID));
		}
		else if(!m_srcMolInfo){
			checkInfo = false;
			MSG_ERROR(UtilStrFormat("SourceInfo Elem is undefined in %s Elem(id=%d)", m_elemName, m_elemID));
		}
		else{
			checkInfo = false;
			MSG_ERROR(UtilStrFormat("DistinationInfo Elem is undefined in %s Elem(id=%d)", m_elemName, m_elemID));
		}
	}

	return checkInfo;
}

/////////////////////////////////////////////////////////////////////////////
// print log
//
bool WarpInfo::printLog(stringstream& wout, int ts, int nid)
{
	int i;

	// base
	WarpBase::printLog(wout,ts,nid);

	ts += WARP_TS;

	WarpMolInfo* molInfoList[4] = {m_srcMolInfo,m_dstMolInfo,m_srcTrigger,m_dstTrigger};
	for( i=0;i<4;i++ )
	{
		if( !molInfoList[i] ) continue;

		if( !molInfoList[i]->printLog(wout,ts,nid) )
		{
			return false;
		}
	}

	return true;
}

