/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */
#include "WarpInfo.h"
#include "FortranFuncWarp.h"

/////////////////////////////////////////////////////////////////////////////
// Calc Capillary
//
bool WarpInfo::CalcCapillary(WarpTemporaryArea *tempArea)
{
	// get elem ID
	int elemID = GetElemID();
	DB1(MESSAGE(UtilStrFormat("WarpInfo::CalcCapillary elemID = %d",elemID)));

	// get equation
	string srcEqnName = m_srcMolInfo? m_srcMolInfo->GetEquation()->name: string("NULL");
	string dstEqnName = m_dstMolInfo? m_dstMolInfo->GetEquation()->name: string("NULL");
	DB1(MESSAGE(UtilStrFormat("  Source Equation      = %s", srcEqnName.c_str())));
	DB1(MESSAGE(UtilStrFormat("  Distination Equation = %s", dstEqnName.c_str())));


	// delta T
	SKL_REAL dt = RicsGetDeltaT();

	// diffusion coefficient
	SKL_REAL coef = GetRealParam("DiffCoef");

	// length of point to point
	SKL_REAL length = GetRealParam("DiffLength");

	// cross-sectional area
	SKL_REAL area = GetRealParam("CrossSectionalArea");

	// length zero check
	if(length == SKL_REAL(0.0)){
		MSG_ERROR(UtilStrFormat("WarpInfo_CalcCapillary DiffLength=%.6e",length));
		return false;
	}

	// volume
	SKL_REAL srcVol = m_srcMolInfo->GetAreaVolume();
	SKL_REAL dstVol = m_dstMolInfo->GetAreaVolume();
	if( srcVol != dstVol )
	{
		MESSAGE(UtilStrFormat("src and dist volume is unmatched"));
	}

	// mol
	SKL_REAL srcMol = m_srcMolInfo->GetAreaMol();
	SKL_REAL dstMol = m_dstMolInfo->GetAreaMol();

	// density
	SKL_REAL srcDen = srcMol / srcVol;
	SKL_REAL dstDen = dstMol / dstVol;

	// Flux
	SKL_REAL srcFlux = coef * (dstDen-srcDen) / length;
	SKL_REAL dstFlux = coef * (srcDen-dstDen) / length;

	// diffusion
	SKL_REAL srcMolNew = srcMol + dt * srcFlux * area;
	SKL_REAL dstMolNew = dstMol + dt * dstFlux * area;
	// SKL_REAL srcMolNew = srcMol + dt * coef * (dstMol-srcMol) / (length*length);
	// SKL_REAL dstMolNew = dstMol + dt * coef * (srcMol-dstMol) / (length*length);

	// changedMol
	SKL_REAL srcMolDiff = (srcMolNew-srcMol)/dt;
	SKL_REAL dstMolDiff = (dstMolNew-dstMol)/dt;

	// calc source term
	m_srcMolInfo->CalcSourceTermByMol(srcMolDiff);
	m_dstMolInfo->CalcSourceTermByMol(dstMolDiff);

DB1(MESSAGE(UtilStrFormat("    mol(src, dst)    = %.6e, %.6e",srcMol,dstMol)));
DB1(MESSAGE(UtilStrFormat("    volume(src, dst) = %.6e, %.6e",srcVol,dstVol)));
DB1(MESSAGE(UtilStrFormat("    coef             = %.6e",coef)));
DB1(MESSAGE(UtilStrFormat("    leng             = %.6e",length)));
DB1(MESSAGE(UtilStrFormat("    area             = %.6e",area)));
DB1(MESSAGE(UtilStrFormat("    molnew(src, dst) = %.6e, %.6e",srcMolNew,dstMolNew)));

	return true;
}

