/*
 * Copyright (c) RIKEN, Japan. All right reserved. 2008-2009
 *
 */

#ifndef __WARP_MOL_INFO_H__
#define __WARP_MOL_INFO_H__

#include "WarpBase.h"

class WarpMolInfo : public WarpBase
{
protected:

// member variables

	// id
	const Equation*  m_eqn; //equation
	const Material*  m_mat; //material
	const Molecular* m_mol; //molecular

	// array
	S4D_REAL* m_vf;  //volume of fluid by material
	S4D_REAL* m_P0;  //molecular density by equation
	S4D_REAL* m_src; //source term by equation

	// index list
	WarpIndexList  m_indexList;    // index list(for own domain)
	WarpIndexList  m_indexListOrg; // index list(original)

// member functions
	bool EquationInitialize();
	SKL_REAL GetAreaDensityMax(bool debugWrite=true);
	SKL_REAL GetAreaDensityMin();
	SKL_REAL GetAreaDensityAve();
	bool UpdateDomainIndex();
	bool LoadPositionSPH();

public:

// member functions
  WarpMolInfo(const CfgInfo& _cfg);
  virtual ~WarpMolInfo();
	bool ParseXML(const CfgElem* elemRoot);
	virtual bool printLog(stringstream& wout, int ts, int nid=-1);

	S_WARP_INDEX GetIndex(int idx);
	size_t GetIndexCount();
	SKL_REAL GetAreaVolume();
	SKL_REAL GetAreaMol();
	SKL_REAL GetAreaDensity(E_WARP_OP flag);
	SKL_REAL CalcSourceTermByDensityRatio(SKL_REAL densityRatio,E_WARP_OP op);
	SKL_REAL CalcSourceTermByDensity(SKL_REAL density);
	SKL_REAL CalcSourceTermByMol(SKL_REAL mol);

	bool     CheckDensityThreshold(SKL_REAL threshold, E_WARP_OP flag1, E_WARP_CMP flag2, WarpTemporaryArea *tempArea);

	bool CheckTrigger(WarpTemporaryArea *tempArea=NULL);

	const Equation* GetEquation(){ return m_eqn; };
	const Material* GetMaterial(){ return m_mat; };
	const Molecular* GetMolecular(){ return m_mol; };

	bool CalcDirect(WarpTemporaryArea *tempArea);
	bool CalcMediation(WarpTemporaryArea *tempArea);
	bool CalcCapillary(WarpTemporaryArea *tempArea);
	bool CalcUser(WarpTemporaryArea *tempArea);

	E_WARP_MOLINFO_TYPE GetType();
};

#endif  /*__WARP_MOL_INFO_H__*/

