//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include "UserDefinedFunction.h"

extern RealPairVector table;

// write user defined function implementation, here
void UserDefinedFunctionInitialize()
{
	DB2(MESSAGE("UserDefinedFunctionInitialize"));
	
#ifdef _SAMPLE_RECEPTER_
	int I, J, K;
	RicsGetVoxelWidth(&I,&J,&K);
	for (int k = 0;   k < K; k++) {
	for (int j = 0;   j < J; j++) {
	for (int i = I/2; i < I; i++) {
		RicsSetPhi("M2","Pi",0.0,i,j,k);
	}}}
#endif

	//////////////////////////////////////////////////
	// for K, set molecular concentration.
	// MESSAGE("UserDefine : SetMolecularConcentration function", 0);
	int elemNumMax = 10;
	for( int en = 0; en < elemNumMax; en++ ){
		//////////////////////////////////////////////////
		// get parameter
		string rootElem;
		stringstream ss;
		ss << "SetMolConc" << en;
		rootElem = ss.str();

		string sEqn, sMol, sPos;
		sEqn = rootElem + "@equation";
		sMol = rootElem + "@molconc";
		sPos = rootElem + "@position";

		string sIsPos, sJsPos, sKsPos;
		string sIePos, sJePos, sKePos;
		sIsPos = sPos + "@si";
		sJsPos = sPos + "@sj";
		sKsPos = sPos + "@sk";
		sIePos = sPos + "@ei";
		sJePos = sPos + "@ej";
		sKePos = sPos + "@ek";

		const char* eqn  = RicsGetExtParamString( sEqn.c_str() );
		SKL_REAL molconc = RicsGetExtParamReal( sMol.c_str() );

		int si = RicsGetExtParamInt( sIsPos.c_str() );
		int sj = RicsGetExtParamInt( sJsPos.c_str() );
		int sk = RicsGetExtParamInt( sKsPos.c_str() );
		int ei = RicsGetExtParamInt( sIePos.c_str() );
		int ej = RicsGetExtParamInt( sJePos.c_str() );
		int ek = RicsGetExtParamInt( sKePos.c_str() );

		if( !eqn || !molconc ){ continue; }

		//////////////////////////////////////////////////
		// exchange index
		int tmpIndex;
		if( ei < si ){
			tmpIndex = si;
			si = ei;
			ei = tmpIndex;
		}
		if( ej < sj ){
			tmpIndex = sj;
			sj = ej;
			ej = tmpIndex;
		}
		if( ek < sk ){
			tmpIndex = sk;
			sk = ek;
			ek = tmpIndex;
		}

		//////////////////////////////////////////////////
		// output log
//		MESSAGE(UtilStrFormat(" %s", rootElem.c_str()));
//		MESSAGE(UtilStrFormat("  Equation : %s, MolConc : %e", eqn, molconc));
//		MESSAGE(UtilStrFormat("  index(%d - %d, %d - %d, %d - %d)", si, ei, sj, ej, sk, ek));

		const Equation* gm = UtilGetEquationId( eqn );

		S4DRN p0 = UtilGetCfg()->S4Dmap[UtilStrFormat("%s", gm->name.c_str())];

		int i, j, k;
		//////////////////////////////////////////////////
		// set molecular concentration : index
		for( k = sk; k <= ek; k++ ){
		for( j = sj; j <= ej; j++ ){
		for( i = si; i <= ei; i++ ){
			int zi, zj, zk;
			const bool in = UtilCheckIdxPdiv(&zi, &zj, &zk, i, j, k, 1);

			if( !in ) { continue; }

			p0.ary->val(zi,zj,zk,gm->id) = molconc;
		}}}
	}
}

