//Copyright (C) 2013 The Computational Science Resarch Program (CSRP), RIKEN
//All rights reserved.
//
//This file is part of RIKEN Integrated Cell Simulator for the K-Computer(RICS-K).
//
//RICS-K is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//RICS-K is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with RICS-K.  If not, see <http://www.gnu.org/licenses/>.


#include "UserDefinedFunction.h"

RealPairVector table;

// write user defined function implementation, here
void UserDefinedFunctionLoop()
{
#ifdef SPHERE175
	const CfgInfo*    cfg      = UtilGetCfg();
	SklParaComponent* para_cmp = cfg->solver->SklGetParaComponent();
	SklParaManager*   para_mng = para_cmp->GetParaManager();
#endif

#ifdef SPHERE175
	const int nID = (para_cmp->IsParallel())? para_cmp->GetMyID(0): 0;
#else
	const int nID = (ParaMngr.IsParallel())? ParaMngr.GetMyID(0): 0;
#endif

#ifdef _SAMPLE_GLC2HX_
	const SKL_REAL ct = RicsGetExtParamReal("GLC_Close_Time");

	static bool f = true;

	if ((t > ct) && f) {
		int I, J, K;
		RicsGetVoxelWidth(&I,&J,&K);
		for (int k = 0; k < K; k++) {
		for (int j = 0; j < J; j++) {
		for (int i = 0; i < I; i++) {
			RicsSetMembraneSwCha("M1:M2","GLC",1,0.0,i,j,k);
		}}}
		MESSAGE("Ch_GLC:M1:M2 closed");
		f = !f;
	}
#endif

#ifdef _MEASURE_SYSTEM_MEMORY_
	SKL_REAL time     = RicsGetCurrentTime();
	unsigned int step = RicsGetCurrentStep();
	
	//----------------------------------
	// sample of RicsGetSystemMemoryInfo
	//----------------------------------
	SystemMemoryInfo minfo;
	if(RicsGetSystemMemoryInfo(minfo) == 1){
		RicsSaveSystemMemoryInfo("sysmem-", "in UDFL", minfo);
//		MESSAGE("-------------------------");
//		MESSAGE("System Memory Info");
//		MESSAGE((UtilStrFormat("ID=%d, pid=%d, step=%d, time=%lf",
//							   info.ID, info.pid, step, time)));
//		MESSAGE((UtilStrFormat("VmPeak: %8d [kB]",info.VmPeak)));
//		MESSAGE((UtilStrFormat("VmSize: %8d [kB]",info.VmSize)));
//		MESSAGE((UtilStrFormat("VmHWM:  %8d [kB]",info.VmHWM)));
//		MESSAGE((UtilStrFormat("VmRSS:  %8d [kB]",info.VmRSS)));
//		MESSAGE((UtilStrFormat("Share:  %8d [kB]",info.Shared)));
//		MESSAGE("-------------------------");
	} 
#endif //_MEASURE_SYSTEM_MEMORY_

	//----------------------------
	// sample of RicsCheckPhiValue
	//----------------------------	
	// prepare for sample
	CheckValueInfo vinfo;
//  TestSetPhi("M1", "Pi", SKL_REAL(-1.0),       1, 2, 3);
//  TestSetPhi("M1", "Pi", SKL_REAL(REAL_MAX*2), 4, 5, 6);
//  TestSetPhi("M1", "Pi", SKL_REAL(101.0),      7, 8, 9);
//	if(RicsCheckPhiValue("M1", "Pi", vinfo, -100.0, 100.0) == 1){
//		RicsSaveCheckValueInfo("test-", "in UDFL", vinfo);
//	
//		int noverflow = vinfo.overflow.size();
//		int nnegative = vinfo.negative.size();
//		int noutrange = vinfo.outrange.size();
//
//		MESSAGE("-----------------------------");
//		MESSAGE(UtilStrFormat("Eq. %s:%s", vinfo.mat.c_str(),vinfo.mol.c_str()));
//		MESSAGE(UtilStrFormat("min phi=%+e (%3d,%3d,%3d) vof=%+e mol=%+e",
//							  vinfo.min.phi,
//							  vinfo.min.ijk[0],
//							  vinfo.min.ijk[1],
//							  vinfo.min.ijk[2],
//							  vinfo.min.vof,
//							  vinfo.min.mol));
//
//		MESSAGE(UtilStrFormat("max phi=%+e (%3d,%3d,%3d) vof=%+e mol=%+e",
//							  vinfo.max.phi,
//							  vinfo.max.ijk[0],
//							  vinfo.max.ijk[1],
//							  vinfo.max.ijk[2],
//							  vinfo.max.vof,
//							  vinfo.max.mol));
//
//		if(noverflow != 0)
//			MESSAGE(UtilStrFormat("%d overflow voxel found:", noverflow));
//		for(int i = 0; i < noverflow; i++){
//			const CheckValueElem& elem = vinfo.overflow[i];
//			MESSAGE(UtilStrFormat("%4d, phi=%+e (%3d,%3d,%3d) vof=%+e mol=%+e",
//								  i,
//								  elem.phi,
//								  elem.ijk[0],
//								  elem.ijk[1],
//								  elem.ijk[2],
//								  elem.vof,
//								  elem.mol));
//		}
//		
//		if(nnegative != 0)
//			MESSAGE(UtilStrFormat("%d negative voxel found:", nnegative));
//		for(int i = 0; i < nnegative; i++){
//			const CheckValueElem& elem = vinfo.negative[i];
//			MESSAGE(UtilStrFormat("%4d, phi=%+e (%3d,%3d,%3d) vof=%+e mol=%+e",
//								  i,
//								  elem.phi,
//								  elem.ijk[0],
//								  elem.ijk[1],
//								  elem.ijk[2],
//								  elem.vof,
//								  elem.mol));
//		}
//
//		if(noutrange != 0)
//			MESSAGE(UtilStrFormat("%d out of range voxel found:", noutrange));
//		for(int i = 0; i < noutrange; i++){
//			const CheckValueElem& elem = vinfo.outrange[i];
//			MESSAGE(UtilStrFormat("%4d, phi=%+e (%3d,%3d,%3d) vof=%+e mol=%+e",
//								  i,
//								  elem.phi,
//								  elem.ijk[0],
//								  elem.ijk[1],
//								  elem.ijk[2],
//								  elem.vof,
//								  elem.mol));
//		}
//
//		MESSAGE("-----------------------------");
//	}else{
//		MESSAGE("RicsCheckPhiValue returns error");
//	}

	//
	// sample for RicsGetTotalNumberOfMolecules
	//
	SKL_REAL total;
//	RicsGetTotalNumberOfMolecules("M2", "GLC", &total);
//	DB2(MESSAGE(UtilStrFormat("total molecules GLC in M2==%e [pieces]", total)));
	
//	RicsGetTotalMoleNumber("M2", "GLC", &total);
//	DB2(MESSAGE(UtilStrFormat("total molecules GLC in M2==%e [mole]", total)));

}
